"""
Copyright 2013 Steven Diamond

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import cvxpy.interface as intf
import numpy as np
import scipy.sparse as sp
from cvxpy.tests.base_test import BaseTest


class TestInterfaces(BaseTest):
    """ Unit tests for matrix interfaces. """

    def setUp(self):
        pass

    def sign_for_intf(self, interface):
        """Test sign for a given interface.
        """
        mat = interface.const_to_matrix([[1, 2, 3, 4], [3, 4, 5, 6]])
        self.assertEqual(intf.sign(mat), (True, False))  # Positive.
        self.assertEqual(intf.sign(-mat), (False, True))  # Negative.
        self.assertEqual(intf.sign(0*mat), (True, True))  # Zero.
        mat = interface.const_to_matrix([[-1, 2, 3, 4], [3, 4, 5, 6]])
        self.assertEqual(intf.sign(mat), (False, False))  # Unknown.

    # # Test cvxopt dense interface.
    # def test_cvxopt_dense(self):
    #     interface = intf.get_matrix_interface(cvxopt.matrix)
    #     # const_to_matrix
    #     mat = interface.const_to_matrix([1, 2, 3])
    #     self.assertEqual(interface.shape(mat), (3, 1))
    #     sp_mat = sp.coo_matrix(([1, 2], ([3, 4], [2, 1])), (5, 5))
    #     mat = interface.const_to_matrix(sp_mat)
    #     self.assertEqual(interface.shape(mat), (5, 5))
    #     # identity
    #     mat = interface.identity(4)
    #     cmp_mat = interface.const_to_matrix(np.eye(4))
    #     self.assertEqual(type(mat), type(cmp_mat))
    #     self.assertEqual(interface.shape(mat), interface.shape(cmp_mat))
    #     assert not mat - cmp_mat
    #     # scalar_matrix
    #     mat = interface.scalar_matrix(2, 4, 3)
    #     self.assertEqual(interface.shape(mat), (4, 3))
    #     self.assertEqual(interface.index(mat, (1, 2)), 2)
    #     # reshape
    #     mat = interface.const_to_matrix([[1, 2, 3], [3, 4, 5]])
    #     mat = interface.reshape(mat, (6, 1))
    #     self.assertEqual(interface.index(mat, (4, 0)), 4)
    #     mat = interface.const_to_matrix(1, convert_scalars=True)
    #     self.assertEqual(type(interface.reshape(mat, (1, 1))), type(mat))
    #     # index
    #     mat = interface.const_to_matrix([[1, 2, 3, 4], [3, 4, 5, 6]])
    #     self.assertEqual(interface.index(mat, (0, 1)), 3)
    #     mat = interface.index(mat, (slice(1, 4, 2), slice(0, 2, None)))
    #     self.assertEqual(list(mat), [2, 4, 4, 6])
    #     # Sign
    #     self.sign_for_intf(interface)

    # # Test cvxopt sparse interface.
    # def test_cvxopt_sparse(self):
    #     interface = intf.get_matrix_interface(cvxopt.spmatrix)
    #     # const_to_matrix
    #     mat = interface.const_to_matrix([1, 2, 3])
    #     self.assertEqual(interface.shape(mat), (3, 1))
    #     # identity
    #     mat = interface.identity(4)
    #     cmp_mat = interface.const_to_matrix(np.eye(4))
    #     self.assertEqual(interface.shape(mat), interface.shape(cmp_mat))
    #     assert not mat - cmp_mat
    #     assert intf.is_sparse(mat)
    #     # scalar_matrix
    #     mat = interface.scalar_matrix(2, 4, 3)
    #     self.assertEqual(interface.shape(mat), (4, 3))
    #     self.assertEqual(interface.index(mat, (1, 2)), 2)
    #     # reshape
    #     mat = interface.const_to_matrix([[1, 2, 3], [3, 4, 5]])
    #     mat = interface.reshape(mat, (6, 1))
    #     self.assertEqual(interface.index(mat, (4, 0)), 4)
    #     mat = interface.const_to_matrix(1, convert_scalars=True)
    #     self.assertEqual(type(interface.reshape(mat, (1, 1))), type(mat))
    #     # Test scalars.
    #     scalar = interface.scalar_matrix(1, 1, 1)
    #     self.assertEqual(type(scalar), cvxopt.spmatrix)
    #     scalar = interface.scalar_matrix(1, 1, 3)
    #     self.assertEqual(scalar.shape, (1, 3))
    #     # index
    #     mat = interface.const_to_matrix([[1, 2, 3, 4], [3, 4, 5, 6]])
    #     self.assertEqual(interface.index(mat, (0, 1)), 3)
    #     mat = interface.index(mat, (slice(1, 4, 2), slice(0, 2, None)))
    #     self.assertEqual(list(mat), [2, 4, 4, 6])
    #     # Sign
    #     self.sign_for_intf(interface)

    # Test numpy ndarray interface.
    def test_ndarray(self):
        interface = intf.get_matrix_interface(np.ndarray)
        # const_to_matrix
        mat = interface.const_to_matrix([1, 2, 3])
        self.assertEqual(interface.shape(mat), (3,))
        mat = interface.const_to_matrix([1, 2])
        self.assertEqual(interface.shape(mat), (2,))
        # # CVXOPT sparse conversion
        # tmp = intf.get_matrix_interface(cvxopt.spmatrix).const_to_matrix([1, 2, 3])
        # mat = interface.const_to_matrix(tmp)
        # assert (mat == interface.const_to_matrix([1, 2, 3])).all()
        # # identity
        # mat = interface.identity(4)
        # cvxopt_dense = intf.get_matrix_interface(cvxopt.matrix)
        # cmp_mat = interface.const_to_matrix(cvxopt_dense.identity(4))
        # self.assertEqual(interface.shape(mat), interface.shape(cmp_mat))
        # assert (mat == cmp_mat).all()
        # scalar_matrix
        mat = interface.scalar_matrix(2, (4, 3))
        self.assertEqual(interface.shape(mat), (4, 3))
        self.assertEqual(interface.index(mat, (1, 2)), 2)
        # reshape
        mat = interface.const_to_matrix([[1, 2, 3], [3, 4, 5]])
        mat = interface.reshape(mat, (6, 1))
        self.assertEqual(interface.index(mat, (4, 0)), 4)
        mat = interface.const_to_matrix(1, convert_scalars=True)
        self.assertEqual(type(interface.reshape(mat, (1, 1))), type(mat))
        # index
        mat = interface.const_to_matrix([[1, 2, 3, 4], [3, 4, 5, 6]])
        self.assertEqual(interface.index(mat, (0, 1)), 3)
        mat = interface.index(mat, (slice(1, 4, 2), slice(0, 2, None)))
        self.assertEqual(list(mat.flatten('C')), [2, 4, 4, 6])
        # Scalars and matrices.
        scalar = interface.const_to_matrix(2)
        mat = interface.const_to_matrix([1, 2, 3])
        assert (scalar*mat == interface.const_to_matrix([2, 4, 6])).all()
        assert (scalar - mat == interface.const_to_matrix([1, 0, -1])).all()
        # Sign
        self.sign_for_intf(interface)
        # shape.
        assert interface.shape(np.array([1, 2, 3])) == (3,)

    # Test numpy matrix interface.
    def test_numpy_matrix(self):
        interface = intf.get_matrix_interface(np.matrix)
        # const_to_matrix
        mat = interface.const_to_matrix([1, 2, 3])
        self.assertEqual(interface.shape(mat), (3, 1))
        mat = interface.const_to_matrix([[1], [2], [3]])
        self.assertEqual(mat[0, 0], 1)
        # identity
        # mat = interface.identity(4)
        # cvxopt_dense = intf.get_matrix_interface(cvxopt.matrix)
        # cmp_mat = interface.const_to_matrix(cvxopt_dense.identity(4))
        # self.assertEqual(interface.shape(mat), interface.shape(cmp_mat))
        # assert not (mat - cmp_mat).any()
        # scalar_matrix
        mat = interface.scalar_matrix(2, (4, 3))
        self.assertEqual(interface.shape(mat), (4, 3))
        self.assertEqual(interface.index(mat, (1, 2)), 2)
        # reshape
        mat = interface.const_to_matrix([[1, 2, 3], [3, 4, 5]])
        mat = interface.reshape(mat, (6, 1))
        self.assertEqual(interface.index(mat, (4, 0)), 4)
        mat = interface.const_to_matrix(1, convert_scalars=True)
        self.assertEqual(type(interface.reshape(mat, (1, 1))), type(mat))
        # index
        mat = interface.const_to_matrix([[1, 2, 3, 4], [3, 4, 5, 6]])
        self.assertEqual(interface.index(mat, (0, 1)), 3)
        mat = interface.index(mat, (slice(1, 4, 2), slice(0, 2, None)))
        assert not (mat - np.array([[2, 4], [4, 6]])).any()
        # Sign
        self.sign_for_intf(interface)

    def test_scipy_sparse(self):
        """Test cvxopt sparse interface.
        """
        interface = intf.get_matrix_interface(sp.csc_matrix)
        # const_to_matrix
        mat = interface.const_to_matrix([1, 2, 3])
        self.assertEqual(interface.shape(mat), (3, 1))
        # C = cvxopt.spmatrix([1, 1, 1, 1, 1], [0, 1, 2, 0, 0, ], [0, 0, 0, 1, 2])
        # mat = interface.const_to_matrix(C)
        # self.assertEqual(interface.shape(mat), (3, 3))
        # identity
        mat = interface.identity(4)
        cmp_mat = interface.const_to_matrix(np.eye(4))
        self.assertEqual(interface.shape(mat), interface.shape(cmp_mat))
        assert (mat - cmp_mat).nnz == 0
        # scalar_matrix
        mat = interface.scalar_matrix(2, (4, 3))
        self.assertEqual(interface.shape(mat), (4, 3))
        self.assertEqual(interface.index(mat, (1, 2)), 2)
        # reshape
        mat = interface.const_to_matrix([[1, 2, 3], [3, 4, 5]])
        mat = interface.reshape(mat, (6, 1))
        self.assertEqual(interface.index(mat, (4, 0)), 4)
        # Test scalars.
        scalar = interface.scalar_matrix(1, (1, 1))
        self.assertEqual(type(scalar), np.ndarray)
        scalar = interface.scalar_matrix(1, (1, 3))
        self.assertEqual(scalar.shape, (1, 3))
        # index
        mat = interface.const_to_matrix([[1, 2, 3, 4], [3, 4, 5, 6]])
        self.assertEqual(interface.index(mat, (0, 1)), 3)
        mat = interface.index(mat, (slice(1, 4, 2), slice(0, 2, None)))
        assert not (mat - np.array([[2, 4], [4, 6]])).any()
        # scalar value
        mat = sp.eye(1)
        self.assertEqual(intf.scalar_value(mat), 1.0)
        # Sign
        self.sign_for_intf(interface)
        # Complex
        # define sparse matrix [[0, 1j],[-1j,0]]
        row = np.array([0, 1])
        col = np.array([1, 0])
        data = np.array([1j, -1j])
        A = sp.csr_matrix((data, (row, col)), shape=(2, 2))
        mat = interface.const_to_matrix(A)
        self.assertEqual(mat[0, 1], 1j)
        self.assertEqual(mat[1, 0], -1j)

    def test_conversion_between_intf(self):
        """Test conversion between every pair of interfaces.
        """
        interfaces = [intf.get_matrix_interface(np.ndarray),
                      intf.get_matrix_interface(np.matrix),
                      intf.get_matrix_interface(sp.csc_matrix)]
        cmp_mat = [[1, 2, 3, 4], [3, 4, 5, 6], [-1, 0, 2, 4]]
        for i in range(len(interfaces)):
            for j in range(i+1, len(interfaces)):
                intf1 = interfaces[i]
                mat1 = intf1.const_to_matrix(cmp_mat)
                intf2 = interfaces[j]
                mat2 = intf2.const_to_matrix(cmp_mat)
                for col in range(len(cmp_mat)):
                    for row in range(len(cmp_mat[0])):
                        key = (slice(row, row+1, None),
                               slice(col, col+1, None))
                        self.assertEqual(intf1.index(mat1, key),
                                         intf2.index(mat2, key))
                        # Convert between the interfaces.
                        self.assertEqual(cmp_mat[col][row],
                                         intf1.index(intf1.const_to_matrix(mat2), key))
                        self.assertEqual(intf2.index(intf2.const_to_matrix(mat1), key),
                                         cmp_mat[col][row])



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.8
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.
from sys import version_info as _swig_python_version_info
if _swig_python_version_info >= (2, 7, 0):
    def swig_import_helper():
        import importlib
        pkg = __name__.rpartition('.')[0]
        mname = '.'.join((pkg, '_cvxcore')).lstrip('.')
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module('_cvxcore')
    _cvxcore = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_cvxcore', [dirname(__file__)])
        except ImportError:
            import _cvxcore
            return _cvxcore
        try:
            _mod = imp.load_module('_cvxcore', fp, pathname, description)
        finally:
            if fp is not None:
                fp.close()
        return _mod
    _cvxcore = swig_import_helper()
    del swig_import_helper
else:
    import _cvxcore
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.


def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr_nondynamic(self, class_type, name, static=1):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    if (not static):
        return object.__getattr__(self, name)
    else:
        raise AttributeError(name)

def _swig_getattr(self, class_type, name):
    return _swig_getattr_nondynamic(self, class_type, name, 0)


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object:
        pass
    _newclass = 0


class SwigPyIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SwigPyIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SwigPyIterator, name)

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _cvxcore.delete_SwigPyIterator
    __del__ = lambda self: None

    def value(self):
        return _cvxcore.SwigPyIterator_value(self)

    def incr(self, n=1):
        return _cvxcore.SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        return _cvxcore.SwigPyIterator_decr(self, n)

    def distance(self, x):
        return _cvxcore.SwigPyIterator_distance(self, x)

    def equal(self, x):
        return _cvxcore.SwigPyIterator_equal(self, x)

    def copy(self):
        return _cvxcore.SwigPyIterator_copy(self)

    def __next__(self):
        return _cvxcore.SwigPyIterator_next(self)

    def __next__(self):
        return _cvxcore.SwigPyIterator___next__(self)

    def previous(self):
        return _cvxcore.SwigPyIterator_previous(self)

    def advance(self, n):
        return _cvxcore.SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        return _cvxcore.SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        return _cvxcore.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        return _cvxcore.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        return _cvxcore.SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        return _cvxcore.SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        return _cvxcore.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self
SwigPyIterator_swigregister = _cvxcore.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)


_cvxcore.VARIABLE_swigconstant(_cvxcore)
VARIABLE = _cvxcore.VARIABLE

_cvxcore.PROMOTE_swigconstant(_cvxcore)
PROMOTE = _cvxcore.PROMOTE

_cvxcore.MUL_swigconstant(_cvxcore)
MUL = _cvxcore.MUL

_cvxcore.RMUL_swigconstant(_cvxcore)
RMUL = _cvxcore.RMUL

_cvxcore.MUL_ELEM_swigconstant(_cvxcore)
MUL_ELEM = _cvxcore.MUL_ELEM

_cvxcore.DIV_swigconstant(_cvxcore)
DIV = _cvxcore.DIV

_cvxcore.SUM_swigconstant(_cvxcore)
SUM = _cvxcore.SUM

_cvxcore.NEG_swigconstant(_cvxcore)
NEG = _cvxcore.NEG

_cvxcore.INDEX_swigconstant(_cvxcore)
INDEX = _cvxcore.INDEX

_cvxcore.TRANSPOSE_swigconstant(_cvxcore)
TRANSPOSE = _cvxcore.TRANSPOSE

_cvxcore.SUM_ENTRIES_swigconstant(_cvxcore)
SUM_ENTRIES = _cvxcore.SUM_ENTRIES

_cvxcore.TRACE_swigconstant(_cvxcore)
TRACE = _cvxcore.TRACE

_cvxcore.RESHAPE_swigconstant(_cvxcore)
RESHAPE = _cvxcore.RESHAPE

_cvxcore.DIAG_VEC_swigconstant(_cvxcore)
DIAG_VEC = _cvxcore.DIAG_VEC

_cvxcore.DIAG_MAT_swigconstant(_cvxcore)
DIAG_MAT = _cvxcore.DIAG_MAT

_cvxcore.UPPER_TRI_swigconstant(_cvxcore)
UPPER_TRI = _cvxcore.UPPER_TRI

_cvxcore.CONV_swigconstant(_cvxcore)
CONV = _cvxcore.CONV

_cvxcore.HSTACK_swigconstant(_cvxcore)
HSTACK = _cvxcore.HSTACK

_cvxcore.VSTACK_swigconstant(_cvxcore)
VSTACK = _cvxcore.VSTACK

_cvxcore.SCALAR_CONST_swigconstant(_cvxcore)
SCALAR_CONST = _cvxcore.SCALAR_CONST

_cvxcore.DENSE_CONST_swigconstant(_cvxcore)
DENSE_CONST = _cvxcore.DENSE_CONST

_cvxcore.SPARSE_CONST_swigconstant(_cvxcore)
SPARSE_CONST = _cvxcore.SPARSE_CONST

_cvxcore.NO_OP_swigconstant(_cvxcore)
NO_OP = _cvxcore.NO_OP

_cvxcore.KRON_swigconstant(_cvxcore)
KRON = _cvxcore.KRON
class LinOp(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, LinOp, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, LinOp, name)
    __repr__ = _swig_repr
    __swig_setmethods__["type"] = _cvxcore.LinOp_type_set
    __swig_getmethods__["type"] = _cvxcore.LinOp_type_get
    if _newclass:
        type = _swig_property(_cvxcore.LinOp_type_get, _cvxcore.LinOp_type_set)
    __swig_setmethods__["size"] = _cvxcore.LinOp_size_set
    __swig_getmethods__["size"] = _cvxcore.LinOp_size_get
    if _newclass:
        size = _swig_property(_cvxcore.LinOp_size_get, _cvxcore.LinOp_size_set)
    __swig_setmethods__["args"] = _cvxcore.LinOp_args_set
    __swig_getmethods__["args"] = _cvxcore.LinOp_args_get
    if _newclass:
        args = _swig_property(_cvxcore.LinOp_args_get, _cvxcore.LinOp_args_set)
    __swig_setmethods__["data_ndim"] = _cvxcore.LinOp_data_ndim_set
    __swig_getmethods__["data_ndim"] = _cvxcore.LinOp_data_ndim_get
    if _newclass:
        data_ndim = _swig_property(_cvxcore.LinOp_data_ndim_get, _cvxcore.LinOp_data_ndim_set)
    __swig_setmethods__["sparse"] = _cvxcore.LinOp_sparse_set
    __swig_getmethods__["sparse"] = _cvxcore.LinOp_sparse_get
    if _newclass:
        sparse = _swig_property(_cvxcore.LinOp_sparse_get, _cvxcore.LinOp_sparse_set)
    __swig_setmethods__["sparse_data"] = _cvxcore.LinOp_sparse_data_set
    __swig_getmethods__["sparse_data"] = _cvxcore.LinOp_sparse_data_get
    if _newclass:
        sparse_data = _swig_property(_cvxcore.LinOp_sparse_data_get, _cvxcore.LinOp_sparse_data_set)
    __swig_setmethods__["dense_data"] = _cvxcore.LinOp_dense_data_set
    __swig_getmethods__["dense_data"] = _cvxcore.LinOp_dense_data_get
    if _newclass:
        dense_data = _swig_property(_cvxcore.LinOp_dense_data_get, _cvxcore.LinOp_dense_data_set)
    __swig_setmethods__["slice"] = _cvxcore.LinOp_slice_set
    __swig_getmethods__["slice"] = _cvxcore.LinOp_slice_get
    if _newclass:
        slice = _swig_property(_cvxcore.LinOp_slice_get, _cvxcore.LinOp_slice_set)

    def __init__(self):
        this = _cvxcore.new_LinOp()
        try:
            self.this.append(this)
        except Exception:
            self.this = this

    def has_constant_type(self):
        return _cvxcore.LinOp_has_constant_type(self)

    def set_dense_data(self, matrix):
        return _cvxcore.LinOp_set_dense_data(self, matrix)

    def set_sparse_data(self, data, row_idxs, col_idxs, rows, cols):
        return _cvxcore.LinOp_set_sparse_data(self, data, row_idxs, col_idxs, rows, cols)
    __swig_destroy__ = _cvxcore.delete_LinOp
    __del__ = lambda self: None
LinOp_swigregister = _cvxcore.LinOp_swigregister
LinOp_swigregister(LinOp)
cvar = _cvxcore.cvar
CONSTANT_ID = cvar.CONSTANT_ID

class ProblemData(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, ProblemData, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, ProblemData, name)
    __repr__ = _swig_repr
    __swig_setmethods__["V"] = _cvxcore.ProblemData_V_set
    __swig_getmethods__["V"] = _cvxcore.ProblemData_V_get
    if _newclass:
        V = _swig_property(_cvxcore.ProblemData_V_get, _cvxcore.ProblemData_V_set)
    __swig_setmethods__["I"] = _cvxcore.ProblemData_I_set
    __swig_getmethods__["I"] = _cvxcore.ProblemData_I_get
    if _newclass:
        I = _swig_property(_cvxcore.ProblemData_I_get, _cvxcore.ProblemData_I_set)
    __swig_setmethods__["J"] = _cvxcore.ProblemData_J_set
    __swig_getmethods__["J"] = _cvxcore.ProblemData_J_get
    if _newclass:
        J = _swig_property(_cvxcore.ProblemData_J_get, _cvxcore.ProblemData_J_set)
    __swig_setmethods__["const_vec"] = _cvxcore.ProblemData_const_vec_set
    __swig_getmethods__["const_vec"] = _cvxcore.ProblemData_const_vec_get
    if _newclass:
        const_vec = _swig_property(_cvxcore.ProblemData_const_vec_get, _cvxcore.ProblemData_const_vec_set)
    __swig_setmethods__["id_to_col"] = _cvxcore.ProblemData_id_to_col_set
    __swig_getmethods__["id_to_col"] = _cvxcore.ProblemData_id_to_col_get
    if _newclass:
        id_to_col = _swig_property(_cvxcore.ProblemData_id_to_col_get, _cvxcore.ProblemData_id_to_col_set)
    __swig_setmethods__["const_to_row"] = _cvxcore.ProblemData_const_to_row_set
    __swig_getmethods__["const_to_row"] = _cvxcore.ProblemData_const_to_row_get
    if _newclass:
        const_to_row = _swig_property(_cvxcore.ProblemData_const_to_row_get, _cvxcore.ProblemData_const_to_row_set)

    def getV(self, values):
        return _cvxcore.ProblemData_getV(self, values)

    def getI(self, values):
        return _cvxcore.ProblemData_getI(self, values)

    def getJ(self, values):
        return _cvxcore.ProblemData_getJ(self, values)

    def getConstVec(self, values):
        return _cvxcore.ProblemData_getConstVec(self, values)

    def __init__(self):
        this = _cvxcore.new_ProblemData()
        try:
            self.this.append(this)
        except Exception:
            self.this = this
    __swig_destroy__ = _cvxcore.delete_ProblemData
    __del__ = lambda self: None
ProblemData_swigregister = _cvxcore.ProblemData_swigregister
ProblemData_swigregister(ProblemData)

class IntVector(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, IntVector, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, IntVector, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _cvxcore.IntVector_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __bool__(self):
        return _cvxcore.IntVector___nonzero__(self)

    def __bool__(self):
        return _cvxcore.IntVector___bool__(self)

    def __len__(self):
        return _cvxcore.IntVector___len__(self)

    def __getslice__(self, i, j):
        return _cvxcore.IntVector___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _cvxcore.IntVector___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _cvxcore.IntVector___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _cvxcore.IntVector___delitem__(self, *args)

    def __getitem__(self, *args):
        return _cvxcore.IntVector___getitem__(self, *args)

    def __setitem__(self, *args):
        return _cvxcore.IntVector___setitem__(self, *args)

    def pop(self):
        return _cvxcore.IntVector_pop(self)

    def append(self, x):
        return _cvxcore.IntVector_append(self, x)

    def empty(self):
        return _cvxcore.IntVector_empty(self)

    def size(self):
        return _cvxcore.IntVector_size(self)

    def swap(self, v):
        return _cvxcore.IntVector_swap(self, v)

    def begin(self):
        return _cvxcore.IntVector_begin(self)

    def end(self):
        return _cvxcore.IntVector_end(self)

    def rbegin(self):
        return _cvxcore.IntVector_rbegin(self)

    def rend(self):
        return _cvxcore.IntVector_rend(self)

    def clear(self):
        return _cvxcore.IntVector_clear(self)

    def get_allocator(self):
        return _cvxcore.IntVector_get_allocator(self)

    def pop_back(self):
        return _cvxcore.IntVector_pop_back(self)

    def erase(self, *args):
        return _cvxcore.IntVector_erase(self, *args)

    def __init__(self, *args):
        this = _cvxcore.new_IntVector(*args)
        try:
            self.this.append(this)
        except Exception:
            self.this = this

    def push_back(self, x):
        return _cvxcore.IntVector_push_back(self, x)

    def front(self):
        return _cvxcore.IntVector_front(self)

    def back(self):
        return _cvxcore.IntVector_back(self)

    def assign(self, n, x):
        return _cvxcore.IntVector_assign(self, n, x)

    def resize(self, *args):
        return _cvxcore.IntVector_resize(self, *args)

    def insert(self, *args):
        return _cvxcore.IntVector_insert(self, *args)

    def reserve(self, n):
        return _cvxcore.IntVector_reserve(self, n)

    def capacity(self):
        return _cvxcore.IntVector_capacity(self)
    __swig_destroy__ = _cvxcore.delete_IntVector
    __del__ = lambda self: None
IntVector_swigregister = _cvxcore.IntVector_swigregister
IntVector_swigregister(IntVector)

class DoubleVector(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, DoubleVector, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, DoubleVector, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _cvxcore.DoubleVector_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __bool__(self):
        return _cvxcore.DoubleVector___nonzero__(self)

    def __bool__(self):
        return _cvxcore.DoubleVector___bool__(self)

    def __len__(self):
        return _cvxcore.DoubleVector___len__(self)

    def __getslice__(self, i, j):
        return _cvxcore.DoubleVector___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _cvxcore.DoubleVector___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _cvxcore.DoubleVector___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _cvxcore.DoubleVector___delitem__(self, *args)

    def __getitem__(self, *args):
        return _cvxcore.DoubleVector___getitem__(self, *args)

    def __setitem__(self, *args):
        return _cvxcore.DoubleVector___setitem__(self, *args)

    def pop(self):
        return _cvxcore.DoubleVector_pop(self)

    def append(self, x):
        return _cvxcore.DoubleVector_append(self, x)

    def empty(self):
        return _cvxcore.DoubleVector_empty(self)

    def size(self):
        return _cvxcore.DoubleVector_size(self)

    def swap(self, v):
        return _cvxcore.DoubleVector_swap(self, v)

    def begin(self):
        return _cvxcore.DoubleVector_begin(self)

    def end(self):
        return _cvxcore.DoubleVector_end(self)

    def rbegin(self):
        return _cvxcore.DoubleVector_rbegin(self)

    def rend(self):
        return _cvxcore.DoubleVector_rend(self)

    def clear(self):
        return _cvxcore.DoubleVector_clear(self)

    def get_allocator(self):
        return _cvxcore.DoubleVector_get_allocator(self)

    def pop_back(self):
        return _cvxcore.DoubleVector_pop_back(self)

    def erase(self, *args):
        return _cvxcore.DoubleVector_erase(self, *args)

    def __init__(self, *args):
        this = _cvxcore.new_DoubleVector(*args)
        try:
            self.this.append(this)
        except Exception:
            self.this = this

    def push_back(self, x):
        return _cvxcore.DoubleVector_push_back(self, x)

    def front(self):
        return _cvxcore.DoubleVector_front(self)

    def back(self):
        return _cvxcore.DoubleVector_back(self)

    def assign(self, n, x):
        return _cvxcore.DoubleVector_assign(self, n, x)

    def resize(self, *args):
        return _cvxcore.DoubleVector_resize(self, *args)

    def insert(self, *args):
        return _cvxcore.DoubleVector_insert(self, *args)

    def reserve(self, n):
        return _cvxcore.DoubleVector_reserve(self, n)

    def capacity(self):
        return _cvxcore.DoubleVector_capacity(self)
    __swig_destroy__ = _cvxcore.delete_DoubleVector
    __del__ = lambda self: None
DoubleVector_swigregister = _cvxcore.DoubleVector_swigregister
DoubleVector_swigregister(DoubleVector)

class IntVector2D(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, IntVector2D, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, IntVector2D, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _cvxcore.IntVector2D_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __bool__(self):
        return _cvxcore.IntVector2D___nonzero__(self)

    def __bool__(self):
        return _cvxcore.IntVector2D___bool__(self)

    def __len__(self):
        return _cvxcore.IntVector2D___len__(self)

    def __getslice__(self, i, j):
        return _cvxcore.IntVector2D___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _cvxcore.IntVector2D___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _cvxcore.IntVector2D___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _cvxcore.IntVector2D___delitem__(self, *args)

    def __getitem__(self, *args):
        return _cvxcore.IntVector2D___getitem__(self, *args)

    def __setitem__(self, *args):
        return _cvxcore.IntVector2D___setitem__(self, *args)

    def pop(self):
        return _cvxcore.IntVector2D_pop(self)

    def append(self, x):
        return _cvxcore.IntVector2D_append(self, x)

    def empty(self):
        return _cvxcore.IntVector2D_empty(self)

    def size(self):
        return _cvxcore.IntVector2D_size(self)

    def swap(self, v):
        return _cvxcore.IntVector2D_swap(self, v)

    def begin(self):
        return _cvxcore.IntVector2D_begin(self)

    def end(self):
        return _cvxcore.IntVector2D_end(self)

    def rbegin(self):
        return _cvxcore.IntVector2D_rbegin(self)

    def rend(self):
        return _cvxcore.IntVector2D_rend(self)

    def clear(self):
        return _cvxcore.IntVector2D_clear(self)

    def get_allocator(self):
        return _cvxcore.IntVector2D_get_allocator(self)

    def pop_back(self):
        return _cvxcore.IntVector2D_pop_back(self)

    def erase(self, *args):
        return _cvxcore.IntVector2D_erase(self, *args)

    def __init__(self, *args):
        this = _cvxcore.new_IntVector2D(*args)
        try:
            self.this.append(this)
        except Exception:
            self.this = this

    def push_back(self, x):
        return _cvxcore.IntVector2D_push_back(self, x)

    def front(self):
        return _cvxcore.IntVector2D_front(self)

    def back(self):
        return _cvxcore.IntVector2D_back(self)

    def assign(self, n, x):
        return _cvxcore.IntVector2D_assign(self, n, x)

    def resize(self, *args):
        return _cvxcore.IntVector2D_resize(self, *args)

    def insert(self, *args):
        return _cvxcore.IntVector2D_insert(self, *args)

    def reserve(self, n):
        return _cvxcore.IntVector2D_reserve(self, n)

    def capacity(self):
        return _cvxcore.IntVector2D_capacity(self)
    __swig_destroy__ = _cvxcore.delete_IntVector2D
    __del__ = lambda self: None
IntVector2D_swigregister = _cvxcore.IntVector2D_swigregister
IntVector2D_swigregister(IntVector2D)

class DoubleVector2D(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, DoubleVector2D, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, DoubleVector2D, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _cvxcore.DoubleVector2D_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __bool__(self):
        return _cvxcore.DoubleVector2D___nonzero__(self)

    def __bool__(self):
        return _cvxcore.DoubleVector2D___bool__(self)

    def __len__(self):
        return _cvxcore.DoubleVector2D___len__(self)

    def __getslice__(self, i, j):
        return _cvxcore.DoubleVector2D___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _cvxcore.DoubleVector2D___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _cvxcore.DoubleVector2D___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _cvxcore.DoubleVector2D___delitem__(self, *args)

    def __getitem__(self, *args):
        return _cvxcore.DoubleVector2D___getitem__(self, *args)

    def __setitem__(self, *args):
        return _cvxcore.DoubleVector2D___setitem__(self, *args)

    def pop(self):
        return _cvxcore.DoubleVector2D_pop(self)

    def append(self, x):
        return _cvxcore.DoubleVector2D_append(self, x)

    def empty(self):
        return _cvxcore.DoubleVector2D_empty(self)

    def size(self):
        return _cvxcore.DoubleVector2D_size(self)

    def swap(self, v):
        return _cvxcore.DoubleVector2D_swap(self, v)

    def begin(self):
        return _cvxcore.DoubleVector2D_begin(self)

    def end(self):
        return _cvxcore.DoubleVector2D_end(self)

    def rbegin(self):
        return _cvxcore.DoubleVector2D_rbegin(self)

    def rend(self):
        return _cvxcore.DoubleVector2D_rend(self)

    def clear(self):
        return _cvxcore.DoubleVector2D_clear(self)

    def get_allocator(self):
        return _cvxcore.DoubleVector2D_get_allocator(self)

    def pop_back(self):
        return _cvxcore.DoubleVector2D_pop_back(self)

    def erase(self, *args):
        return _cvxcore.DoubleVector2D_erase(self, *args)

    def __init__(self, *args):
        this = _cvxcore.new_DoubleVector2D(*args)
        try:
            self.this.append(this)
        except Exception:
            self.this = this

    def push_back(self, x):
        return _cvxcore.DoubleVector2D_push_back(self, x)

    def front(self):
        return _cvxcore.DoubleVector2D_front(self)

    def back(self):
        return _cvxcore.DoubleVector2D_back(self)

    def assign(self, n, x):
        return _cvxcore.DoubleVector2D_assign(self, n, x)

    def resize(self, *args):
        return _cvxcore.DoubleVector2D_resize(self, *args)

    def insert(self, *args):
        return _cvxcore.DoubleVector2D_insert(self, *args)

    def reserve(self, n):
        return _cvxcore.DoubleVector2D_reserve(self, n)

    def capacity(self):
        return _cvxcore.DoubleVector2D_capacity(self)
    __swig_destroy__ = _cvxcore.delete_DoubleVector2D
    __del__ = lambda self: None
DoubleVector2D_swigregister = _cvxcore.DoubleVector2D_swigregister
DoubleVector2D_swigregister(DoubleVector2D)

class IntIntMap(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, IntIntMap, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, IntIntMap, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _cvxcore.IntIntMap_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __bool__(self):
        return _cvxcore.IntIntMap___nonzero__(self)

    def __bool__(self):
        return _cvxcore.IntIntMap___bool__(self)

    def __len__(self):
        return _cvxcore.IntIntMap___len__(self)
    def __iter__(self):
        return self.key_iterator()
    def iterkeys(self):
        return self.key_iterator()
    def itervalues(self):
        return self.value_iterator()
    def iteritems(self):
        return self.iterator()

    def __getitem__(self, key):
        return _cvxcore.IntIntMap___getitem__(self, key)

    def __delitem__(self, key):
        return _cvxcore.IntIntMap___delitem__(self, key)

    def has_key(self, key):
        return _cvxcore.IntIntMap_has_key(self, key)

    def keys(self):
        return _cvxcore.IntIntMap_keys(self)

    def values(self):
        return _cvxcore.IntIntMap_values(self)

    def items(self):
        return _cvxcore.IntIntMap_items(self)

    def __contains__(self, key):
        return _cvxcore.IntIntMap___contains__(self, key)

    def key_iterator(self):
        return _cvxcore.IntIntMap_key_iterator(self)

    def value_iterator(self):
        return _cvxcore.IntIntMap_value_iterator(self)

    def __setitem__(self, *args):
        return _cvxcore.IntIntMap___setitem__(self, *args)

    def asdict(self):
        return _cvxcore.IntIntMap_asdict(self)

    def __init__(self, *args):
        this = _cvxcore.new_IntIntMap(*args)
        try:
            self.this.append(this)
        except Exception:
            self.this = this

    def empty(self):
        return _cvxcore.IntIntMap_empty(self)

    def size(self):
        return _cvxcore.IntIntMap_size(self)

    def swap(self, v):
        return _cvxcore.IntIntMap_swap(self, v)

    def begin(self):
        return _cvxcore.IntIntMap_begin(self)

    def end(self):
        return _cvxcore.IntIntMap_end(self)

    def rbegin(self):
        return _cvxcore.IntIntMap_rbegin(self)

    def rend(self):
        return _cvxcore.IntIntMap_rend(self)

    def clear(self):
        return _cvxcore.IntIntMap_clear(self)

    def get_allocator(self):
        return _cvxcore.IntIntMap_get_allocator(self)

    def count(self, x):
        return _cvxcore.IntIntMap_count(self, x)

    def erase(self, *args):
        return _cvxcore.IntIntMap_erase(self, *args)

    def find(self, x):
        return _cvxcore.IntIntMap_find(self, x)

    def lower_bound(self, x):
        return _cvxcore.IntIntMap_lower_bound(self, x)

    def upper_bound(self, x):
        return _cvxcore.IntIntMap_upper_bound(self, x)
    __swig_destroy__ = _cvxcore.delete_IntIntMap
    __del__ = lambda self: None
IntIntMap_swigregister = _cvxcore.IntIntMap_swigregister
IntIntMap_swigregister(IntIntMap)

class LinOpVector(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, LinOpVector, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, LinOpVector, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _cvxcore.LinOpVector_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __bool__(self):
        return _cvxcore.LinOpVector___nonzero__(self)

    def __bool__(self):
        return _cvxcore.LinOpVector___bool__(self)

    def __len__(self):
        return _cvxcore.LinOpVector___len__(self)

    def __getslice__(self, i, j):
        return _cvxcore.LinOpVector___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _cvxcore.LinOpVector___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _cvxcore.LinOpVector___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _cvxcore.LinOpVector___delitem__(self, *args)

    def __getitem__(self, *args):
        return _cvxcore.LinOpVector___getitem__(self, *args)

    def __setitem__(self, *args):
        return _cvxcore.LinOpVector___setitem__(self, *args)

    def pop(self):
        return _cvxcore.LinOpVector_pop(self)

    def append(self, x):
        return _cvxcore.LinOpVector_append(self, x)

    def empty(self):
        return _cvxcore.LinOpVector_empty(self)

    def size(self):
        return _cvxcore.LinOpVector_size(self)

    def swap(self, v):
        return _cvxcore.LinOpVector_swap(self, v)

    def begin(self):
        return _cvxcore.LinOpVector_begin(self)

    def end(self):
        return _cvxcore.LinOpVector_end(self)

    def rbegin(self):
        return _cvxcore.LinOpVector_rbegin(self)

    def rend(self):
        return _cvxcore.LinOpVector_rend(self)

    def clear(self):
        return _cvxcore.LinOpVector_clear(self)

    def get_allocator(self):
        return _cvxcore.LinOpVector_get_allocator(self)

    def pop_back(self):
        return _cvxcore.LinOpVector_pop_back(self)

    def erase(self, *args):
        return _cvxcore.LinOpVector_erase(self, *args)

    def __init__(self, *args):
        this = _cvxcore.new_LinOpVector(*args)
        try:
            self.this.append(this)
        except Exception:
            self.this = this

    def push_back(self, x):
        return _cvxcore.LinOpVector_push_back(self, x)

    def front(self):
        return _cvxcore.LinOpVector_front(self)

    def back(self):
        return _cvxcore.LinOpVector_back(self)

    def assign(self, n, x):
        return _cvxcore.LinOpVector_assign(self, n, x)

    def resize(self, *args):
        return _cvxcore.LinOpVector_resize(self, *args)

    def insert(self, *args):
        return _cvxcore.LinOpVector_insert(self, *args)

    def reserve(self, n):
        return _cvxcore.LinOpVector_reserve(self, n)

    def capacity(self):
        return _cvxcore.LinOpVector_capacity(self)
    __swig_destroy__ = _cvxcore.delete_LinOpVector
    __del__ = lambda self: None
LinOpVector_swigregister = _cvxcore.LinOpVector_swigregister
LinOpVector_swigregister(LinOpVector)


def build_matrix(*args):
    return _cvxcore.build_matrix(*args)
build_matrix = _cvxcore.build_matrix
# This file is compatible with both classic and new-style classes.





import cvxpy.settings as s
from cvxpy.reductions.solvers import utilities
import cvxpy.interface as intf
from cvxpy.reductions import Solution
from cvxpy.reductions.solvers.qp_solvers.qp_solver import QpSolver
import numpy as np


def constrain_gurobi_infty(v):
    '''
    Limit values of vector v between +/- infinity as
    defined in the Gurobi package
    '''
    import gurobipy as grb
    n = len(v)

    for i in range(n):
        if v[i] >= 1e20:
            v[i] = grb.GRB.INFINITY
        if v[i] <= -1e20:
            v[i] = -grb.GRB.INFINITY


class GUROBI(QpSolver):
    """QP interface for the Gurobi solver"""

    MIP_CAPABLE = True

    # Map of Gurobi status to CVXPY status.
    STATUS_MAP = {2: s.OPTIMAL,
                  3: s.INFEASIBLE,
                  5: s.UNBOUNDED,
                  4: s.INFEASIBLE_INACCURATE,
                  6: s.INFEASIBLE,
                  7: s.SOLVER_ERROR,
                  8: s.SOLVER_ERROR,
                  # TODO could be anything.
                  # means time expired.
                  9: s.SOLVER_ERROR,
                  10: s.SOLVER_ERROR,
                  11: s.SOLVER_ERROR,
                  12: s.SOLVER_ERROR,
                  13: s.OPTIMAL_INACCURATE}

    def name(self):
        return s.GUROBI

    def import_solver(self):
        import gurobipy
        gurobipy

    def invert(self, results, inverse_data):
        model = results["model"]
        x_grb = model.getVars()
        n = len(x_grb)
        constraints_grb = model.getConstrs()
        m = len(constraints_grb)

        # Start populating attribute dictionary
        attr = {s.SOLVE_TIME: model.Runtime,
                s.NUM_ITERS: model.BarIterCount}

        # Map GUROBI statuses back to CVXPY statuses
        status = self.STATUS_MAP.get(model.Status, s.SOLVER_ERROR)

        if status in s.SOLUTION_PRESENT:
            opt_val = model.objVal
            x = np.array([x_grb[i].X for i in range(n)])

            primal_vars = {
                list(inverse_data.id_map.keys())[0]:
                intf.DEFAULT_INTF.const_to_matrix(np.array(x))
            }

            # Only add duals if not a MIP.
            dual_vars = None
            if not inverse_data.is_mip:
                y = -np.array([constraints_grb[i].Pi for i in range(m)])

                dual_vars = utilities.get_dual_values(
                    intf.DEFAULT_INTF.const_to_matrix(y),
                    utilities.extract_dual_value,
                    inverse_data.sorted_constraints)
        else:
            primal_vars = None
            dual_vars = None
            opt_val = np.inf
            if status == s.UNBOUNDED:
                opt_val = -np.inf

        return Solution(status, opt_val, primal_vars, dual_vars, attr)

    def solve_via_data(self, data, warm_start, verbose, solver_opts, solver_cache=None):
        import gurobipy as grb
        # N.B. Here we assume that the matrices in data are in csc format
        P = data[s.P]
        q = data[s.Q]
        A = data[s.A].tocsr()       # Convert A matrix to csr format
        b = data[s.B]
        F = data[s.F].tocsr()       # Convert F matrix to csr format
        g = data[s.G]
        n = data['n_var']

        # Constrain values between bounds
        constrain_gurobi_infty(b)
        constrain_gurobi_infty(g)

        # Create a new model
        model = grb.Model()

        # Add variables
        vtypes = {}
        for ind in data[s.BOOL_IDX]:
            vtypes[ind] = grb.GRB.BINARY
        for ind in data[s.INT_IDX]:
            vtypes[ind] = grb.GRB.INTEGER
        for i in range(n):
            if i not in vtypes:
                vtypes[i] = grb.GRB.CONTINUOUS
        model.addVars(int(n),
                      ub={i: grb.GRB.INFINITY for i in range(n)},
                      lb={i: -grb.GRB.INFINITY for i in range(n)},
                      vtype=vtypes)
        model.update()
        x = np.array(model.getVars(), copy=False)

        if A.shape[0] > 0:
            if hasattr(model, '_v811_addMConstrs'):
                # We can pass all of A == b at once
                sense = np.repeat(grb.GRB.EQUAL, A.shape[0])
                model._v811_addMConstrs(A, sense, b)
            else:
                # Add equality constraints: iterate over the rows of A
                # adding each row into the model
                for i in range(A.shape[0]):
                    start = A.indptr[i]
                    end = A.indptr[i+1]
                    variables = x[A.indices[start:end]]
                    coeff = A.data[start:end]
                    expr = grb.LinExpr(coeff, variables)
                    model.addConstr(expr, grb.GRB.EQUAL, b[i])
        model.update()

        if F.shape[0] > 0:
            if hasattr(model, '_v811_addMConstrs'):
                # We can pass all of F <= g at once
                sense = np.repeat(grb.GRB.LESS_EQUAL, F.shape[0])
                model._v811_addMConstrs(F, sense, g)
            else:
                # Add inequality constraints: iterate over the rows of F
                # adding each row into the model
                for i in range(F.shape[0]):
                    start = F.indptr[i]
                    end = F.indptr[i+1]
                    variables = x[F.indices[start:end]]
                    coeff = F.data[start:end]
                    expr = grb.LinExpr(coeff, variables)
                    model.addConstr(expr, grb.GRB.LESS_EQUAL, g[i])
        model.update()

        # Define objective
        obj = grb.QuadExpr()
        if hasattr(model, '_v811_setMObjective'):
            P = P.tocoo()
            model._v811_setMObjective(0.5 * P, q)
        else:
            if P.count_nonzero():  # If there are any nonzero elms in P
                P = P.tocoo()
                obj.addTerms(0.5*P.data, vars=list(x[P.row]),
                             vars2=list(x[P.col]))
            obj.add(grb.LinExpr(q, x))  # Add linear part
            model.setObjective(obj)  # Set objective
        model.update()

        # Set verbosity and other parameters
        model.setParam("OutputFlag", verbose)
        model.setParam("QCPDual", True)

        for key, value in list(solver_opts.items()):
            model.setParam(key, value)

        # Update model
        model.update()

        # Solve problem
        results_dict = {}
        try:
            # Solve
            model.optimize()
        except Exception:  # Error in the solution
            results_dict["status"] = s.SOLVER_ERROR

        results_dict["model"] = model

        return results_dict



"""
Copyright, the CVXPY authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import cvxpy.lin_ops.lin_utils as lu
# Only need Variable from expressions, but that would create a circular import.
from cvxpy.constraints.constraint import Constraint
import numpy as np


class NonPos(Constraint):
    """A constraint of the form :math:`x \\leq 0`.

    The preferred way of creating a ``NonPos`` constraint is through
    operator overloading. To constrain an expression ``x`` to be non-positive,
    simply write ``x <= 0``; to constrain ``x`` to be non-negative, write
    ``x >= 0``. The former creates a ``NonPos`` constraint with ``x``
    as its argument, while the latter creates one with ``-x`` as its argument.
    Strict inequalities are not supported, as they do not make sense in a
    numerical setting.

    Parameters
    ----------
    expr : Expression
        The expression to constrain.
    constr_id : int
        A unique id for the constraint.
    """
    def __init__(self, expr, constr_id=None):
        super(NonPos, self).__init__([expr], constr_id)

    def name(self):
        return "%s <= 0" % self.args[0]

    def is_dcp(self):
        """A non-positive constraint is DCP if its argument is convex."""
        return self.args[0].is_convex()

    def is_dgp(self):
        return False

    def is_dqcp(self):
        return self.args[0].is_quasiconvex()

    def canonicalize(self):
        """Returns the graph implementation of the object.

        Marks the top level constraint as the dual_holder,
        so the dual value will be saved to the LeqConstraint.

        Returns
        -------
        tuple
            A tuple of (affine expression, [constraints]).
        """
        obj, constraints = self.args[0].canonical_form
        dual_holder = lu.create_leq(obj, constr_id=self.id)
        return (None, constraints + [dual_holder])

    @property
    def residual(self):
        """The residual of the constraint.

        Returns
        ---------
        NumPy.ndarray
        """
        if self.expr.value is None:
            return None
        return np.maximum(self.expr.value, 0)


class Inequality(Constraint):
    """A constraint of the form :math:`x \\leq y`.

    Parameters
    ----------
    expr : Expression
        The expression to constrain.
    constr_id : int
        A unique id for the constraint.
    """
    def __init__(self, lhs, rhs, constr_id=None):
        self._expr = lhs - rhs
        if self._expr.is_complex():
            raise ValueError("Inequality constraints cannot be complex.")
        super(Inequality, self).__init__([lhs, rhs], constr_id)

    def _construct_dual_variables(self, args):
        super(Inequality, self)._construct_dual_variables([self._expr])

    @property
    def expr(self):
        return self._expr

    def name(self):
        return "%s <= %s" % (self.args[0], self.args[1])

    @property
    def shape(self):
        """int : The shape of the constrained expression."""
        return self.expr.shape

    @property
    def size(self):
        """int : The size of the constrained expression."""
        return self.expr.size

    def is_dcp(self):
        """A non-positive constraint is DCP if its argument is convex."""
        return self.expr.is_convex()

    def is_dgp(self):
        return (self.args[0].is_log_log_convex() and
                self.args[1].is_log_log_concave())

    def is_dqcp(self):
        return (
            self.is_dcp() or
            (self.args[0].is_quasiconvex() and self.args[1].is_constant()) or
            (self.args[0].is_constant() and self.args[1].is_quasiconcave()))

    @property
    def residual(self):
        """The residual of the constraint.

        Returns
        ---------
        NumPy.ndarray
        """
        if self.expr.value is None:
            return None
        return np.maximum(self.expr.value, 0)



"""
Copyright 2013 Steven Diamond, 2017 Robin Verschueren

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import cvxpy.interface as intf
import cvxpy.settings as s
from cvxpy.constraints import Zero, NonPos, SOC, PSD
from cvxpy.reductions.solution import failure_solution, Solution
from cvxpy.reductions.solvers import utilities
from cvxpy.reductions.solvers.solver import group_constraints
from cvxpy.reductions.solvers.conic_solvers.ecos_conif import ECOS
from cvxpy.reductions.solvers.conic_solvers.conic_solver import (ConicSolver,
                                                                 ConeDims)
from cvxpy.reductions.solvers.compr_matrix import compress_matrix
from cvxpy.reductions.solvers.kktsolver import get_kktsolver
import scipy.sparse as sp
import scipy
import numpy as np


# Utility method for formatting a ConeDims instance into a dictionary
# that can be supplied to ecos.
def dims_to_solver_dict(cone_dims):
    cones = {
        "l": int(cone_dims.nonpos),
        "q": [int(v) for v in cone_dims.soc],
        "s": [int(v) for v in cone_dims.psd],
    }
    return cones


class CVXOPT(ECOS):
    """An interface for the CVXOPT solver.
    """

    # Solver capabilities.
    MIP_CAPABLE = False
    SUPPORTED_CONSTRAINTS = ConicSolver.SUPPORTED_CONSTRAINTS + [SOC,
                                                                 PSD]

    # Map of CVXOPT status to CVXPY status.
    STATUS_MAP = {"optimal": s.OPTIMAL,
                  "feasible": s.OPTIMAL_INACCURATE,
                  "infeasible": s.INFEASIBLE,
                  "primal infeasible": s.INFEASIBLE,
                  "LP relaxation is primal infeasible": s.INFEASIBLE,
                  "LP relaxation is dual infeasible": s.UNBOUNDED,
                  "unbounded": s.UNBOUNDED,
                  "dual infeasible": s.UNBOUNDED,
                  "unknown": s.SOLVER_ERROR,
                  "undefined": s.SOLVER_ERROR,
                  "solver_error": s.SOLVER_ERROR}

    def name(self):
        """The name of the solver.
        """
        return s.CVXOPT

    def import_solver(self):
        """Imports the solver.
        """
        import cvxopt
        cvxopt  # For flake8

    def accepts(self, problem):
        """Can CVXOPT solve the problem?
        """
        # TODO check if is matrix stuffed.
        if not problem.objective.args[0].is_affine():
            return False
        for constr in problem.constraints:
            if type(constr) not in self.SUPPORTED_CONSTRAINTS:
                return False
            for arg in constr.args:
                if not arg.is_affine():
                    return False
        return True

    def apply(self, problem):
        """Returns a new problem and data for inverting the new solution.

        Returns
        -------
        tuple
            (dict of arguments needed for the solver, inverse data)
        """
        data = {}
        inv_data = {self.VAR_ID: problem.variables()[0].id}
        data[s.C], data[s.OFFSET] = ConicSolver.get_coeff_offset(
            problem.objective.args[0])
        data[s.C] = data[s.C].ravel()
        inv_data[s.OFFSET] = data[s.OFFSET][0]

        constr_map = group_constraints(problem.constraints)
        data[ConicSolver.DIMS] = ConeDims(constr_map)

        inv_data[self.EQ_CONSTR] = constr_map[Zero]
        data[s.A], data[s.B] = self.group_coeff_offset(
            problem, constr_map[Zero], ECOS.EXP_CONE_ORDER)

        # Order and group nonlinear constraints.
        neq_constr = constr_map[NonPos] + constr_map[SOC] + constr_map[PSD]
        inv_data[self.NEQ_CONSTR] = neq_constr
        data[s.G], data[s.H] = self.group_coeff_offset(
            problem, neq_constr, ECOS.EXP_CONE_ORDER)

        return data, inv_data

    def invert(self, solution, inverse_data):
        """Returns the solution to the original problem given the inverse_data.
        """
        status = solution['status']

        primal_vars = None
        dual_vars = None
        if status in s.SOLUTION_PRESENT:
            opt_val = solution['value'] + inverse_data[s.OFFSET]
            primal_vars = {inverse_data[self.VAR_ID]: solution['primal']}
            eq_dual = utilities.get_dual_values(
                solution[s.EQ_DUAL],
                utilities.extract_dual_value,
                inverse_data[self.EQ_CONSTR])
            leq_dual = utilities.get_dual_values(
                solution[s.INEQ_DUAL],
                utilities.extract_dual_value,
                inverse_data[self.NEQ_CONSTR])
            eq_dual.update(leq_dual)
            dual_vars = eq_dual
            return Solution(status, opt_val, primal_vars, dual_vars, {})
        else:
            return failure_solution(status)

    def solve_via_data(self, data, warm_start, verbose, solver_opts, solver_cache=None):
        import cvxopt
        import cvxopt.solvers
        # Save original cvxopt solver options.
        old_options = cvxopt.solvers.options.copy()
        # Save old data in case need to use robust solver.
        data[s.DIMS] = dims_to_solver_dict(data[s.DIMS])
        # User chosen KKT solver option.
        kktsolver = self.get_kktsolver_opt(solver_opts)
        # Cannot have redundant rows unless using robust LDL kktsolver.
        if kktsolver != s.ROBUST_KKTSOLVER:
            # Will detect infeasibility.
            if self.remove_redundant_rows(data) == s.INFEASIBLE:
                return {s.STATUS: s.INFEASIBLE}
        # Convert A, b, G, h, c to CVXOPT matrices.
        data[s.C] = intf.dense2cvxopt(data[s.C])
        var_length = data[s.C].size[0]
        if data[s.A] is None:
            data[s.A] = np.zeros((0, var_length))
            data[s.B] = np.zeros((0, 1))
        data[s.A] = intf.sparse2cvxopt(data[s.A])
        data[s.B] = intf.dense2cvxopt(data[s.B])
        if data[s.G] is None:
            data[s.G] = np.zeros((0, var_length))
            data[s.H] = np.zeros((0, 1))
        data[s.G] = intf.sparse2cvxopt(data[s.G])
        data[s.H] = intf.dense2cvxopt(data[s.H])

        # Apply any user-specific options.
        # Silence solver.
        solver_opts["show_progress"] = verbose
        # Rename max_iters to maxiters.
        if "max_iters" in solver_opts:
            solver_opts["maxiters"] = solver_opts["max_iters"]
        for key, value in list(solver_opts.items()):
            cvxopt.solvers.options[key] = value

        # Always do 1 step of iterative refinement after solving KKT system.
        if "refinement" not in cvxopt.solvers.options:
            cvxopt.solvers.options["refinement"] = 1

        try:
            if kktsolver == s.ROBUST_KKTSOLVER:
                # Get custom kktsolver.
                kktsolver = get_kktsolver(data[s.G],
                                          data[s.DIMS],
                                          data[s.A])
            results_dict = cvxopt.solvers.conelp(data[s.C],
                                                 data[s.G],
                                                 data[s.H],
                                                 data[s.DIMS],
                                                 data[s.A],
                                                 data[s.B],
                                                 kktsolver=kktsolver)
        # Catch exceptions in CVXOPT and convert them to solver errors.
        except ValueError:
            results_dict = {"status": "unknown"}

        # Restore original cvxopt solver options.
        self._restore_solver_options(old_options)

        # Construct solution.
        solution = {}
        status = self.STATUS_MAP[results_dict['status']]
        solution[s.STATUS] = status
        if solution[s.STATUS] in s.SOLUTION_PRESENT:
            primal_val = results_dict['primal objective']
            solution[s.VALUE] = primal_val + data[s.OFFSET]
            solution[s.PRIMAL] = results_dict['x']
            solution[s.EQ_DUAL] = results_dict['y']
            solution[s.INEQ_DUAL] = results_dict['z']
            # Need to multiply duals by Q and P_leq.
            if "Q" in data:
                y = results_dict['y']
                # Test if all constraints eliminated.
                if y.size[0] == 0:
                    dual_len = data["Q"].size[0]
                    solution[s.EQ_DUAL] = cvxopt.matrix(0., (dual_len, 1))
                else:
                    solution[s.EQ_DUAL] = data["Q"]*y
            if "P_leq" in data:
                leq_len = data[s.DIMS][s.LEQ_DIM]
                P_rows = data["P_leq"].size[0]
                new_len = P_rows + solution[s.INEQ_DUAL].size[0] - leq_len
                new_dual = cvxopt.matrix(0., (new_len, 1))
                z = solution[s.INEQ_DUAL][:leq_len]
                # Test if all constraints eliminated.
                if z.size[0] == 0:
                    new_dual[:P_rows] = 0
                else:
                    new_dual[:P_rows] = data["P_leq"] * z
                new_dual[P_rows:] = solution[s.INEQ_DUAL][leq_len:]
                solution[s.INEQ_DUAL] = new_dual

            for key in [s.PRIMAL, s.EQ_DUAL, s.INEQ_DUAL]:
                solution[key] = intf.cvxopt2dense(solution[key])
        return solution

    @staticmethod
    def remove_redundant_rows(data):
        """Remove redundant constraints from A and G.

        Parameters
        ----------
        data : dict
            All the problem data.

        Returns
        -------
        str
            A status indicating if infeasibility was detected.
        """
        # Extract data.
        dims = data[s.DIMS]
        A = data[s.A]
        G = data[s.G]
        b = data[s.B]
        h = data[s.H]
        # Remove redundant rows in A.
        if A is not None:
            # The pivoting improves robustness.
            Q, R, P = scipy.linalg.qr(A.todense(), pivoting=True)
            rows_to_keep = []
            for i in range(R.shape[0]):
                if np.linalg.norm(R[i, :]) > 1e-10:
                    rows_to_keep.append(i)
            R = R[rows_to_keep, :]
            Q = Q[:, rows_to_keep]
            # Invert P from col -> var to var -> col.
            Pinv = np.zeros(P.size, dtype='int')
            for i in range(P.size):
                Pinv[P[i]] = i
            # Rearrage R.
            R = R[:, Pinv]
            A = R
            b_old = b
            b = Q.T.dot(b)
            # If b is not in the range of Q,
            # the problem is infeasible.
            if not np.allclose(b_old, Q.dot(b)):
                return s.INFEASIBLE
            dims[s.EQ_DIM] = int(b.shape[0])
            data["Q"] = intf.dense2cvxopt(Q)
        # Remove obviously redundant rows in G's <= constraints.
        if G is not None:
            G = G.tocsr()
            G_leq = G[:dims[s.LEQ_DIM], :]
            h_leq = h[:dims[s.LEQ_DIM]].ravel()
            G_other = G[dims[s.LEQ_DIM]:, :]
            h_other = h[dims[s.LEQ_DIM]:].ravel()
            G_leq, h_leq, P_leq = compress_matrix(G_leq, h_leq)
            dims[s.LEQ_DIM] = int(h_leq.shape[0])
            data["P_leq"] = intf.sparse2cvxopt(P_leq)
            G = sp.vstack([G_leq, G_other])
            h = np.hstack([h_leq, h_other])
        # Convert A, b, G, h to CVXOPT matrices.
        data[s.A] = A
        data[s.G] = G
        data[s.B] = b
        data[s.H] = h
        return s.OPTIMAL

    @staticmethod
    def _restore_solver_options(old_options):
        import cvxopt.solvers
        for key, value in list(cvxopt.solvers.options.items()):
            if key in old_options:
                cvxopt.solvers.options[key] = old_options[key]
            else:
                del cvxopt.solvers.options[key]

    @staticmethod
    def get_kktsolver_opt(solver_opts):
        """Returns the KKT solver selected by the user.

        Removes the KKT solver from solver_opts.

        Parameters
        ----------
        solver_opts : dict
            Additional arguments for the solver.

        Returns
        -------
        str or None
            The KKT solver chosen by the user.
        """
        if "kktsolver" in solver_opts:
            kktsolver = solver_opts["kktsolver"]
            del solver_opts["kktsolver"]
        else:
            kktsolver = 'chol'
        return kktsolver



import cvxpy
from cvxpy.tests.base_test import BaseTest


class TestDgp(BaseTest):
    def test_product(self):
        x = cvxpy.Variable((), pos=True)
        y = cvxpy.Variable((), pos=True)
        prod = x * y
        self.assertTrue(prod.is_dgp())
        self.assertTrue(prod.is_log_log_convex())
        self.assertTrue(prod.is_log_log_concave())

        prod *= prod
        self.assertTrue(prod.is_dgp())
        self.assertTrue(prod.is_log_log_convex())
        self.assertTrue(prod.is_log_log_concave())

        prod *= 5.0
        self.assertTrue(prod.is_dgp())
        self.assertTrue(prod.is_log_log_convex())
        self.assertTrue(prod.is_log_log_concave())

        prod *= -5.0
        self.assertTrue(not prod.is_dgp())
        self.assertTrue(not prod.is_log_log_convex())
        self.assertTrue(not prod.is_log_log_concave())

    def test_product_with_unconstrained_variables_is_not_dgp(self):
        x = cvxpy.Variable()
        y = cvxpy.Variable()
        prod = x * y
        self.assertTrue(not prod.is_dgp())
        self.assertTrue(not prod.is_log_log_convex())
        self.assertTrue(not prod.is_log_log_concave())

        z = cvxpy.Variable((), pos=True)
        prod = x * z
        self.assertTrue(not prod.is_dgp())
        self.assertTrue(not prod.is_log_log_convex())
        self.assertTrue(not prod.is_log_log_concave())

    def test_division(self):
        x = cvxpy.Variable(pos=True)
        y = cvxpy.Variable(pos=True)
        div = x / y

        self.assertTrue(div.is_log_log_affine())

        posynomial = 5.0 * x * y + 1.2 * y * y
        div = x / y
        self.assertTrue(div.is_log_log_affine())

        div = posynomial / (3.0 * x * y**(-0.1))
        self.assertTrue(div.is_log_log_convex())
        self.assertFalse(div.is_log_log_concave())
        self.assertTrue(div.is_dgp())

        div = posynomial / (3.0 * x + y)
        self.assertFalse(div.is_log_log_convex())
        self.assertFalse(div.is_log_log_concave())
        self.assertFalse(div.is_dgp())

    def test_add(self):
        x = cvxpy.Variable(pos=True)
        y = cvxpy.Variable(pos=True)
        expr = x + y
        self.assertTrue(expr.is_dgp())
        self.assertTrue(expr.is_log_log_convex())
        self.assertTrue(not expr.is_log_log_concave())

        posynomial = 5.0 * x * y + 1.2 * y * y
        self.assertTrue(posynomial.is_dgp())
        self.assertTrue(posynomial.is_log_log_convex())

    def test_add_with_unconstrained_variables_is_not_dgp(self):
        x = cvxpy.Variable()
        y = cvxpy.Variable(pos=True)
        expr = x + y
        self.assertTrue(not expr.is_dgp())
        self.assertTrue(not expr.is_log_log_convex())
        self.assertTrue(not expr.is_log_log_concave())

        posynomial = 5.0 * x * y + 1.2 * y * y
        self.assertTrue(not posynomial.is_dgp())
        self.assertTrue(not posynomial.is_log_log_convex())
        self.assertTrue(not posynomial.is_log_log_concave())

    def test_monomials(self):
        x = cvxpy.Variable(pos=True)
        y = cvxpy.Variable(pos=True)
        z = cvxpy.Variable(pos=True)
        monomial = 5.0 * (x ** 0.1) * y ** (-0.1) * z ** (3)
        self.assertTrue(monomial.is_dgp())
        self.assertTrue(monomial.is_log_log_convex())
        self.assertTrue(monomial.is_log_log_concave())

        monomial *= -1.0
        self.assertTrue(not monomial.is_dgp())
        self.assertTrue(not monomial.is_log_log_convex())
        self.assertTrue(not monomial.is_log_log_concave())

    def test_maximum(self):
        x = cvxpy.Variable(pos=True)
        y = cvxpy.Variable(pos=True)
        z = cvxpy.Variable(pos=True)
        monomial = 5.0 * (x ** 0.1) * y ** (-0.1) * z ** (3)
        posynomial = 5.0 * x * y + 1.2 * y * y
        another_posynomial = posynomial * posynomial
        expr = cvxpy.maximum(monomial, posynomial, another_posynomial)
        self.assertTrue(expr.is_dgp())
        self.assertTrue(expr.is_log_log_convex())
        self.assertTrue(not expr.is_log_log_concave())

        expr = posynomial * expr
        self.assertTrue(expr.is_dgp())
        self.assertTrue(expr.is_log_log_convex())
        self.assertTrue(not expr.is_log_log_concave())

        expr = posynomial * expr + expr
        self.assertTrue(expr.is_dgp())
        self.assertTrue(expr.is_log_log_convex())

    def test_minimum(self):
        x = cvxpy.Variable(pos=True)
        y = cvxpy.Variable(pos=True)
        z = cvxpy.Variable(pos=True)
        monomial = 5.0 * (x ** 0.1) * y ** (-0.1) * z ** (3)
        posynomial = 5.0 * x * y + 1.2 * y * y
        another_posynomial = posynomial * posynomial
        expr = cvxpy.minimum(monomial, 1 / posynomial, 1 / another_posynomial)
        self.assertTrue(expr.is_dgp())
        self.assertTrue(not expr.is_log_log_convex())
        self.assertTrue(expr.is_log_log_concave())

        expr = (1 / posynomial) * expr
        self.assertTrue(expr.is_dgp())
        self.assertTrue(not expr.is_log_log_convex())
        self.assertTrue(expr.is_log_log_concave())

        expr = expr ** 2
        self.assertTrue(expr.is_dgp())
        self.assertTrue(not expr.is_log_log_convex())
        self.assertTrue(expr.is_log_log_concave())

    def test_constant(self):
        x = cvxpy.Constant(1.0)
        self.assertTrue(x.is_dgp())
        self.assertFalse((-1.0*x).is_dgp())

    def test_geo_mean(self):
        x = cvxpy.Variable(3, pos=True)
        p = [1, 2, 0.5]
        geo_mean = cvxpy.geo_mean(x, p)
        self.assertTrue(geo_mean.is_dgp())
        self.assertTrue(geo_mean.is_log_log_affine())
        self.assertTrue(geo_mean.is_log_log_convex())
        self.assertTrue(geo_mean.is_log_log_concave())



"""
Copyright 2013 Steven Diamond

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import cvxpy as cvx
import cvxpy.interface as intf
from cvxpy.tests.base_test import BaseTest
from cvxpy.reductions.solvers.conic_solvers import ecos_conif
import numpy as np
import unittest


class TestExamples(BaseTest):
    """ Unit tests using example problems. """

    # Find the largest Euclidean ball in the polyhedron.
    def test_chebyshev_center(self):
        # The goal is to find the largest Euclidean ball (i.e. its center and
        # radius) that lies in a polyhedron described by linear inequalites in this
        # fashion: P = {x : a_i'*x <= b_i, i=1,...,m} where x is in R^2

        # Generate the input data
        a1 = np.array([2, 1])
        a2 = np.array([2, -1])
        a3 = np.array([-1, 2])
        a4 = np.array([-1, -2])
        b = np.ones(4)

        # Create and solve the model
        r = cvx.Variable(name='r')
        x_c = cvx.Variable(2, name='x_c')
        obj = cvx.Maximize(r)
        constraints = [  # TODO have atoms compute values for constants.
            a1.T*x_c + np.linalg.norm(a1)*r <= b[0],
            a2.T*x_c + np.linalg.norm(a2)*r <= b[1],
            a3.T*x_c + np.linalg.norm(a3)*r <= b[2],
            a4.T*x_c + np.linalg.norm(a4)*r <= b[3],
        ]

        p = cvx.Problem(obj, constraints)
        result = p.solve()
        self.assertAlmostEqual(result, 0.447214)
        self.assertAlmostEqual(r.value, result)
        self.assertItemsAlmostEqual(x_c.value, [0, 0])

    # Test issue with numpy scalars.
    def test_numpy_scalars(self):
        n = 6
        eps = 1e-6
        np.random.seed(10)
        P0 = np.random.randn(n, n)
        eye = np.eye(n)
        P0 = P0.T.dot(P0) + eps * eye

        print(P0)

        P1 = np.random.randn(n, n)
        P1 = P1.T.dot(P1)
        P2 = np.random.randn(n, n)
        P2 = P2.T.dot(P2)
        P3 = np.random.randn(n, n)
        P3 = P3.T.dot(P3)

        q0 = np.random.randn(n, 1)
        q1 = np.random.randn(n, 1)
        q2 = np.random.randn(n, 1)
        q3 = np.random.randn(n, 1)

        r0 = np.random.randn(1, 1)
        r1 = np.random.randn(1, 1)
        r2 = np.random.randn(1, 1)
        r3 = np.random.randn(1, 1)

        slack = cvx.Variable()
        # Form the problem
        x = cvx.Variable(n)
        objective = cvx.Minimize(0.5*cvx.quad_form(x, P0) + q0.T*x + r0 + slack)
        constraints = [0.5*cvx.quad_form(x, P1) + q1.T*x + r1 <= slack,
                       0.5*cvx.quad_form(x, P2) + q2.T*x + r2 <= slack,
                       0.5*cvx.quad_form(x, P3) + q3.T*x + r3 <= slack,
                       ]

        # We now find the primal result and compare it to the dual result
        # to check if strong duality holds i.e. the duality gap is effectively zero
        p = cvx.Problem(objective, constraints)
        p.solve()

        # Note that since our data is random,
        # we may need to run this program multiple times to get a feasible primal
        # When feasible, we can print out the following values
        print(x.value)  # solution
        lam1 = constraints[0].dual_value
        lam2 = constraints[1].dual_value
        lam3 = constraints[2].dual_value
        print(type(lam1))

        P_lam = P0 + lam1*P1 + lam2*P2 + lam3*P3
        q_lam = q0 + lam1*q1 + lam2*q2 + lam3*q3
        r_lam = r0 + lam1*r1 + lam2*r2 + lam3*r3
        dual_result = -0.5*q_lam.T.dot(P_lam).dot(q_lam) + r_lam
        print(dual_result.shape)
        self.assertEqual(intf.shape(dual_result), (1, 1))

    # Tests examples from the README.
    def test_readme_examples(self):
        import numpy
        numpy.random.seed(1)
        # cvx.Problem data.
        m = 30
        n = 20
        A = numpy.random.randn(m, n)
        b = numpy.random.randn(m)

        # Construct the problem.
        x = cvx.Variable(n)
        objective = cvx.Minimize(cvx.sum_squares(A*x - b))
        constraints = [0 <= x, x <= 1]
        p = cvx.Problem(objective, constraints)

        # The optimal objective is returned by p.solve().
        p.solve()
        # The optimal value for x is stored in x.value.
        print(x.value)
        # The optimal Lagrange multiplier for a constraint
        # is stored in constraint.dual_value.
        print(constraints[0].dual_value)

        ####################################################

        # Scalar variable.
        a = cvx.Variable()

        # Column vector variable of length 5.
        x = cvx.Variable(5)

        # Matrix variable with 4 rows and 7 columns.
        A = cvx.Variable((4, 7))

        ####################################################

        # Positive scalar parameter.
        m = cvx.Parameter(nonneg=True)

        # Column vector parameter with unknown sign (by default).
        cvx.Parameter(5)

        # Matrix parameter with negative entries.
        G = cvx.Parameter((4, 7), nonpos=True)

        # Assigns a constant value to G.
        G.value = -numpy.ones((4, 7))

        # Raises an error for assigning a value with invalid sign.
        with self.assertRaises(Exception) as cm:
            G.value = numpy.ones((4, 7))
        self.assertEqual(str(cm.exception), "Parameter value must be nonpositive.")

        ####################################################
        a = cvx.Variable()
        x = cvx.Variable(5)

        # expr is an Expression object after each assignment.
        expr = 2*x
        expr = expr - a
        expr = cvx.sum(expr) + cvx.norm(x, 2)

        ####################################################

        import numpy as np

        # cvx.Problem data.
        n = 10
        m = 5
        A = np.random.randn(n, m)
        b = np.random.randn(n)
        gamma = cvx.Parameter(nonneg=True)

        # Construct the problem.
        x = cvx.Variable(m)
        objective = cvx.Minimize(cvx.sum_squares(A*x - b) + gamma*cvx.norm(x, 1))
        p = cvx.Problem(objective)

        # Assign a value to gamma and find the optimal x.
        def get_x(gamma_value):
            gamma.value = gamma_value
            p.solve()
            return x.value

        gammas = np.logspace(-1, 2, num=2)
        # Serial computation.
        [get_x(value) for value in gammas]

        ####################################################
        n = 10

        mu = np.random.randn(1, n)
        sigma = np.random.randn(n, n)
        sigma = sigma.T.dot(sigma)
        gamma = cvx.Parameter(nonneg=True)
        gamma.value = 1
        x = cvx.Variable(n)

        # Constants:
        # mu is the vector of expected returns.
        # sigma is the covariance matrix.
        # gamma is a cvx.Parameter that trades off risk and return.

        # cvx.Variables:
        # x is a vector of stock holdings as fractions of total assets.

        expected_return = mu*x
        risk = cvx.quad_form(x, sigma)

        objective = cvx.Maximize(expected_return - gamma*risk)
        p = cvx.Problem(objective, [cvx.sum(x) == 1])
        p.solve()

        # The optimal expected return.
        print(expected_return.value)

        # The optimal risk.
        print(risk.value)

        ###########################################

        N = 50
        M = 40
        n = 10
        data = []
        for i in range(N):
            data += [(1, np.random.normal(loc=1.0, scale=2.0, size=n))]
        for i in range(M):
            data += [(-1, np.random.normal(loc=-1.0, scale=2.0, size=n))]

        # Construct problem.
        gamma = cvx.Parameter(nonneg=True)
        gamma.value = 0.1
        # 'a' is a variable constrained to have at most 6 non-zero entries.
        a = cvx.Variable(n)  # mi.SparseVar(n, nonzeros=6)
        b = cvx.Variable()

        slack = [cvx.pos(1 - label*(sample.T*a - b)) for (label, sample) in data]
        objective = cvx.Minimize(cvx.norm(a, 2) + gamma*sum(slack))
        p = cvx.Problem(objective)
        # Extensions can attach new solve methods to the CVXPY cvx.Problem class.
        # p.solve(method="admm")
        p.solve()

        # Count misclassifications.
        errors = 0
        for label, sample in data:
            if label*(sample.T*a - b).value < 0:
                errors += 1

        print("%s misclassifications" % errors)
        print(a.value)
        print(b.value)

    def test_advanced1(self):
        """Code from the advanced tutorial.
        """
        # Solving a problem with different solvers.
        x = cvx.Variable(2)
        obj = cvx.Minimize(x[0] + cvx.norm(x, 1))
        constraints = [x >= 2]
        prob = cvx.Problem(obj, constraints)

        # Solve with ECOS.
        prob.solve(solver=cvx.ECOS)
        print("optimal value with ECOS:", prob.value)
        self.assertAlmostEqual(prob.value, 6)

        # Solve with ECOS_BB.
        prob.solve(solver=cvx.ECOS_BB)
        print("optimal value with ECOS_BB:", prob.value)
        self.assertAlmostEqual(prob.value, 6)

        # Solve with CVXOPT.
        if cvx.CVXOPT in cvx.installed_solvers():
            prob.solve(solver=cvx.CVXOPT)
            print("optimal value with CVXOPT:", prob.value)
            self.assertAlmostEqual(prob.value, 6)

        # Solve with SCS.
        prob.solve(solver=cvx.SCS)
        print("optimal value with SCS:", prob.value)
        self.assertAlmostEqual(prob.value, 6, places=2)

        if cvx.CPLEX in cvx.installed_solvers():
            # Solve with CPLEX.
            prob.solve(solver=cvx.CPLEX)
            print("optimal value with CPLEX:", prob.value)
            self.assertAlmostEqual(prob.value, 6)

        if cvx.GLPK in cvx.installed_solvers():
            # Solve with GLPK.
            prob.solve(solver=cvx.GLPK)
            print("optimal value with GLPK:", prob.value)
            self.assertAlmostEqual(prob.value, 6)

            # Solve with GLPK_MI.
            prob.solve(solver=cvx.GLPK_MI)
            print("optimal value with GLPK_MI:", prob.value)
            self.assertAlmostEqual(prob.value, 6)

        if cvx.GUROBI in cvx.installed_solvers():
            # Solve with Gurobi.
            prob.solve(solver=cvx.GUROBI)
            print("optimal value with GUROBI:", prob.value)
            self.assertAlmostEqual(prob.value, 6)

        print(cvx.installed_solvers())

    def test_log_det(self):
        # Generate data
        x = np.array([[0.55, 0.0],
                      [0.25, 0.35],
                      [-0.2, 0.2],
                      [-0.25, -0.1],
                      [-0.0, -0.3],
                      [0.4, -0.2]]).T
        (n, m) = x.shape

        # Create and solve the model
        A = cvx.Variable((n, n))
        b = cvx.Variable(n)
        obj = cvx.Maximize(cvx.log_det(A))
        constraints = []
        for i in range(m):
            constraints.append(cvx.norm(A*x[:, i] + b) <= 1)
        p = cvx.Problem(obj, constraints)
        result = p.solve()
        self.assertAlmostEqual(result, 1.9746, places=2)

    def test_portfolio_problem(self):
        """Test portfolio problem that caused dcp_attr errors.
        """
        import numpy as np
        import scipy.sparse as sp
        np.random.seed(5)
        n = 100  # 10000
        m = 10  # 100

        F = sp.rand(m, n, density=0.01)
        F.data = np.ones(len(F.data))
        D = sp.eye(n).tocoo()
        D.data = np.random.randn(len(D.data))**2
        Z = np.random.randn(m, 1)
        Z = Z.dot(Z.T)

        x = cvx.Variable(n)
        y = x.__rmul__(F)
        # DCP attr causes error because not all the curvature
        # matrices are reduced to constants when an atom
        # is scalar.
        cvx.square(cvx.norm(D*x)) + cvx.square(Z*y)

    def test_intro(self):
        """Test examples from cvxpy.org introduction.
        """
        import numpy

        # cvx.Problem data.
        m = 30
        n = 20
        numpy.random.seed(1)
        A = numpy.random.randn(m, n)
        b = numpy.random.randn(m)

        # Construct the problem.
        x = cvx.Variable(n)
        objective = cvx.Minimize(cvx.sum_squares(A*x - b))
        constraints = [0 <= x, x <= 1]
        prob = cvx.Problem(objective, constraints)

        # The optimal objective is returned by p.solve().
        prob.solve()
        # The optimal value for x is stored in x.value.
        print(x.value)
        # The optimal Lagrange multiplier for a constraint
        # is stored in constraint.dual_value.
        print(constraints[0].dual_value)

        ########################################

        # Create two scalar variables.
        x = cvx.Variable()
        y = cvx.Variable()

        # Create two constraints.
        constraints = [x + y == 1,
                       x - y >= 1]

        # Form objective.
        obj = cvx.Minimize(cvx.square(x - y))

        # Form and solve problem.
        prob = cvx.Problem(obj, constraints)
        prob.solve()  # Returns the optimal value.
        print("status:", prob.status)
        print("optimal value", prob.value)
        print("optimal var", x.value, y.value)

        ########################################

        # Create two scalar variables.
        x = cvx.Variable()
        y = cvx.Variable()

        # Create two constraints.
        constraints = [x + y == 1,
                       x - y >= 1]

        # Form objective.
        obj = cvx.Minimize(cvx.square(x - y))

        # Form and solve problem.
        prob = cvx.Problem(obj, constraints)
        prob.solve()  # Returns the optimal value.
        print("status:", prob.status)
        print("optimal value", prob.value)
        print("optimal var", x.value, y.value)

        self.assertEqual(prob.status, cvx.OPTIMAL)
        self.assertAlmostEqual(prob.value, 1.0)
        self.assertAlmostEqual(x.value, 1.0)
        self.assertAlmostEqual(y.value, 0)

        ########################################

        # Replace the objective.
        prob = cvx.Problem(cvx.Maximize(x + y), prob.constraints)
        print("optimal value", prob.solve())

        self.assertAlmostEqual(prob.value, 1.0, places=3)

        # Replace the constraint (x + y == 1).
        constraints = prob.constraints
        constraints[0] = (x + y <= 3)
        prob = cvx.Problem(prob.objective, constraints)
        print("optimal value", prob.solve())

        self.assertAlmostEqual(prob.value, 3.0, places=2)

        ########################################

        x = cvx.Variable()

        # An infeasible problem.
        prob = cvx.Problem(cvx.Minimize(x), [x >= 1, x <= 0])
        prob.solve()
        print("status:", prob.status)
        print("optimal value", prob.value)

        self.assertEqual(prob.status, cvx.INFEASIBLE)
        self.assertAlmostEqual(prob.value, np.inf)

        # An unbounded problem.
        prob = cvx.Problem(cvx.Minimize(x))
        prob.solve()
        print("status:", prob.status)
        print("optimal value", prob.value)

        self.assertEqual(prob.status, cvx.UNBOUNDED)
        self.assertAlmostEqual(prob.value, -np.inf)

        ########################################

        # A scalar variable.
        cvx.Variable()

        # Column vector variable of length 5.
        x = cvx.Variable(5)

        # Matrix variable with 4 rows and 7 columns.
        A = cvx.Variable((4, 7))

        ########################################
        import numpy

        # cvx.Problem data.
        m = 10
        n = 5
        numpy.random.seed(1)
        A = numpy.random.randn(m, n)
        b = numpy.random.randn(m)

        # Construct the problem.
        x = cvx.Variable(n)
        objective = cvx.Minimize(cvx.sum_squares(A*x - b))
        constraints = [0 <= x, x <= 1]
        prob = cvx.Problem(objective, constraints)

        print("Optimal value", prob.solve())
        print("Optimal var")
        print(x.value)  # A numpy matrix.

        self.assertAlmostEqual(prob.value, 4.14133859146)

        ########################################
        # Positive scalar parameter.
        m = cvx.Parameter(nonneg=True)

        # Column vector parameter with unknown sign (by default).
        cvx.Parameter(5)

        # Matrix parameter with negative entries.
        G = cvx.Parameter((4, 7), nonpos=True)

        # Assigns a constant value to G.
        G.value = -numpy.ones((4, 7))
        ########################################

        # Create parameter, then assign value.
        rho = cvx.Parameter(nonneg=True)
        rho.value = 2

        # Initialize parameter with a value.
        rho = cvx.Parameter(nonneg=True, value=2)

        ########################################

        import numpy

        # cvx.Problem data.
        n = 15
        m = 10
        numpy.random.seed(1)
        A = numpy.random.randn(n, m)
        b = numpy.random.randn(n)
        # gamma must be positive due to DCP rules.
        gamma = cvx.Parameter(nonneg=True)

        # Construct the problem.
        x = cvx.Variable(m)
        error = cvx.sum_squares(A*x - b)
        obj = cvx.Minimize(error + gamma*cvx.norm(x, 1))
        prob = cvx.Problem(obj)

        # Construct a trade-off curve of ||Ax-b||^2 vs. ||x||_1
        sq_penalty = []
        l1_penalty = []
        x_values = []
        gamma_vals = numpy.logspace(-4, 6)
        for val in gamma_vals:
            gamma.value = val
            prob.solve()
            # Use expr.value to get the numerical value of
            # an expression in the problem.
            sq_penalty.append(error.value)
            l1_penalty.append(cvx.norm(x, 1).value)
            x_values.append(x.value)

        ########################################
        import numpy

        X = cvx.Variable((5, 4))
        A = numpy.ones((3, 5))

        # Use expr.size to get the dimensions.
        print("dimensions of X:", X.size)
        print("dimensions of sum(X):", cvx.sum(X).size)
        print("dimensions of A*X:", (A*X).size)

        # ValueError raised for invalid dimensions.
        try:
            A + X
        except ValueError as e:
            print(e)

    def test_inpainting(self):
        """Test image in-painting.
        """
        import numpy as np
        np.random.seed(1)
        rows, cols = 100, 100
        # Load the images.
        # Convert to arrays.
        Uorig = np.random.randint(0, 255, size=(rows, cols))

        rows, cols = Uorig.shape
        # Known is 1 if the pixel is known,
        # 0 if the pixel was corrupted.
        Known = np.zeros((rows, cols))
        for i in range(rows):
            for j in range(cols):
                if np.random.random() > 0.7:
                    Known[i, j] = 1
        Ucorr = Known*Uorig
        # Recover the original image using total variation in-painting.
        U = cvx.Variable((rows, cols))
        obj = cvx.Minimize(cvx.tv(U))
        constraints = [cvx.multiply(Known, U) == cvx.multiply(Known, Ucorr)]
        prob = cvx.Problem(obj, constraints)
        prob.solve(solver=cvx.SCS)

    def test_advanced2(self):
        """Test code from the advanced section of the tutorial.
        """
        x = cvx.Variable()
        prob = cvx.Problem(cvx.Minimize(cvx.square(x)), [x == 2])
        # Get ECOS arguments.
        data, chain, inverse = prob.get_problem_data(cvx.ECOS)

        # Get ECOS_BB arguments.
        data, chain, inverse = prob.get_problem_data(cvx.ECOS_BB)

        # Get CVXOPT arguments.
        if cvx.CVXOPT in cvx.installed_solvers():
            data, chain, inverse = prob.get_problem_data(cvx.CVXOPT)

        # Get SCS arguments.
        data, chain, inverse = prob.get_problem_data(cvx.SCS)

        import ecos
        # Get ECOS arguments.
        data, chain, inverse = prob.get_problem_data(cvx.ECOS)
        # Call ECOS solver.
        solution = ecos.solve(data["c"], data["G"], data["h"],
                              ecos_conif.dims_to_solver_dict(data["dims"]),
                              data["A"], data["b"])
        # Unpack raw solver output.
        prob.unpack_results(solution, chain, inverse)

    def test_log_sum_exp(self):
        """Test log_sum_exp function that failed in Github issue.
        """
        import numpy as np
        np.random.seed(1)
        m = 5
        n = 2
        X = np.ones((m, n))
        w = cvx.Variable(n)

        expr2 = [cvx.log_sum_exp(cvx.hstack([0, X[i, :]*w])) for i in range(m)]
        expr3 = sum(expr2)
        obj = cvx.Minimize(expr3)
        p = cvx.Problem(obj)
        p.solve(solver=cvx.SCS, max_iters=1)

    # # Risk return tradeoff curve
    # def test_risk_return_tradeoff(self):
    #     from math import sqrt
    #     from cvxopt import matrix
    #     from cvxopt.blas import dot
    #     from cvxopt.solvers import qp, options
    #     import scipy

    #     n = 4
    #     S = matrix( [[ 4e-2,  6e-3, -4e-3,   0.0 ],
    #                  [ 6e-3,  1e-2,  0.0,    0.0 ],
    #                  [-4e-3,  0.0,   2.5e-3, 0.0 ],
    #                  [ 0.0,   0.0,   0.0,    0.0 ]] )
    #     pbar = matrix([.12, .10, .07, .03])

    #     N = 100
    #     # CVXPY
    #     Sroot = numpy.asmatrix(scipy.linalg.sqrtm(S))
    #     x = cvx.Variable(n, name='x')
    #     mu = cvx.Parameter(name='mu')
    #     mu.value = 1 # TODO cvx.Parameter("positive")
    #     objective = cvx.Minimize(-pbar*x + mu*quad_over_lin(Sroot*x,1))
    #     constraints = [sum(x) == 1, x >= 0]
    #     p = cvx.Problem(objective, constraints)

    #     mus = [ 10**(5.0*t/N-1.0) for t in range(N) ]
    #     xs = []
    #     for mu_val in mus:
    #         mu.value = mu_val
    #         p.solve()
    #         xs.append(x.value)
    #     returns = [ dot(pbar,x) for x in xs ]
    #     risks = [ sqrt(dot(x, S*x)) for x in xs ]

    #     # QP solver


if __name__ == '__main__':
    unittest.main()



"""
Copyright, the CVXPY authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import cvxpy as cp
from cvxpy.reductions.solvers import bisection
import cvxpy.settings as s
from cvxpy.tests import base_test

import numpy as np


SOLVER = cp.ECOS


class TestDqcp(base_test.BaseTest):
    def test_basic_with_interval(self):
        x = cp.Variable()
        expr = cp.ceil(x)

        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconvex())
        self.assertTrue(expr.is_quasiconcave())
        self.assertFalse(expr.is_convex())
        self.assertFalse(expr.is_concave())
        self.assertFalse(expr.is_dcp())
        self.assertFalse(expr.is_dgp())

        problem = cp.Problem(cp.Minimize(expr), [x >= 12, x <= 17])
        self.assertTrue(problem.is_dqcp())
        self.assertFalse(problem.is_dcp())
        self.assertFalse(problem.is_dgp())

        red = cp.Dqcp2Dcp(problem)
        reduced = red.reduce()
        self.assertTrue(reduced.is_dcp())
        self.assertEqual(len(reduced.parameters()), 1)
        soln = bisection.bisect(reduced, low=12, high=17)
        self.assertAlmostEqual(soln.opt_val, 12.0, places=3)

        problem.unpack(soln)
        self.assertEqual(soln.opt_val, problem.value)
        self.assertAlmostEqual(x.value, 12.0, places=3)

    def test_basic_without_interval(self):
        x = cp.Variable()
        expr = cp.ceil(x)

        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconvex())
        self.assertTrue(expr.is_quasiconcave())
        self.assertFalse(expr.is_convex())
        self.assertFalse(expr.is_concave())
        self.assertFalse(expr.is_dcp())
        self.assertFalse(expr.is_dgp())

        problem = cp.Problem(cp.Minimize(expr), [x >= 12, x <= 17])
        self.assertTrue(problem.is_dqcp())
        self.assertFalse(problem.is_dcp())
        self.assertFalse(problem.is_dgp())

        red = cp.Dqcp2Dcp(problem)
        reduced = red.reduce()
        self.assertTrue(reduced.is_dcp())
        self.assertEqual(len(reduced.parameters()), 1)
        soln = bisection.bisect(reduced)
        self.assertAlmostEqual(soln.opt_val, 12.0, places=3)

        problem.unpack(soln)
        self.assertEqual(soln.opt_val, problem.value)
        self.assertAlmostEqual(x.value, 12.0, places=3)

    def test_basic_solve(self):
        x = cp.Variable()
        expr = cp.ceil(x)

        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconvex())
        self.assertTrue(expr.is_quasiconcave())
        self.assertFalse(expr.is_convex())
        self.assertFalse(expr.is_concave())
        self.assertFalse(expr.is_dcp())
        self.assertFalse(expr.is_dgp())

        problem = cp.Problem(cp.Minimize(expr), [x >= 12, x <= 17])
        self.assertTrue(problem.is_dqcp())
        self.assertFalse(problem.is_dcp())
        self.assertFalse(problem.is_dgp())
        problem.solve(SOLVER, qcp=True, low=12, high=17)
        self.assertAlmostEqual(problem.value, 12.0, places=3)
        self.assertAlmostEqual(x.value, 12.0, places=3)

        problem._clear_solution()
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.value, 12.0, places=3)
        self.assertAlmostEqual(x.value, 12.0, places=3)

        problem._clear_solution()
        problem.solve(SOLVER, qcp=True, high=17)
        self.assertAlmostEqual(problem.value, 12.0, places=3)
        self.assertAlmostEqual(x.value, 12.0, places=3)

        problem._clear_solution()
        problem.solve(SOLVER, qcp=True, low=12)
        self.assertAlmostEqual(problem.value, 12.0, places=3)
        self.assertAlmostEqual(x.value, 12.0, places=3)

        problem._clear_solution()
        problem.solve(SOLVER, qcp=True, low=0, high=100)
        self.assertAlmostEqual(problem.value, 12.0, places=3)
        self.assertAlmostEqual(x.value, 12.0, places=3)

    def test_basic_maximization_with_interval(self):
        x = cp.Variable()
        expr = cp.ceil(x)

        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconvex())
        self.assertTrue(expr.is_quasiconcave())
        self.assertFalse(expr.is_convex())
        self.assertFalse(expr.is_concave())
        self.assertFalse(expr.is_dcp())
        self.assertFalse(expr.is_dgp())

        problem = cp.Problem(cp.Maximize(expr), [x >= 12, x <= 17])
        self.assertTrue(problem.is_dqcp())
        self.assertFalse(problem.is_dcp())
        self.assertFalse(problem.is_dgp())

        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(x.value, 17.0, places=3)

    def test_basic_maximum(self):
        x, y = cp.Variable(2)
        expr = cp.maximum(cp.ceil(x), cp.ceil(y))

        problem = cp.Problem(cp.Minimize(expr), [x >= 12, x <= 17, y >= 17.4])
        self.assertTrue(problem.is_dqcp())
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.objective.value, 18.0)
        self.assertLess(x.value, 17.1)
        self.assertGreater(x.value, 11.9)
        self.assertGreater(y.value, 17.3)

    def test_basic_minimum(self):
        x, y = cp.Variable(2)
        expr = cp.minimum(cp.ceil(x), cp.ceil(y))

        problem = cp.Problem(cp.Maximize(expr), [x >= 11.9, x <= 15.8, y >= 17.4])
        self.assertTrue(problem.is_dqcp())
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.objective.value, 16.0)
        self.assertLess(x.value, 16.0)
        self.assertGreater(x.value, 14.9)
        self.assertGreater(y.value, 17.3)

    def test_basic_composition(self):
        x, y = cp.Variable(2)
        expr = cp.maximum(cp.ceil(cp.ceil(x)), cp.ceil(cp.ceil(y)))

        problem = cp.Problem(cp.Minimize(expr), [x >= 12, x <= 17, y >= 17.4])
        self.assertTrue(problem.is_dqcp())
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.objective.value, 18.0)
        self.assertLess(x.value, 17.1)
        self.assertGreater(x.value, 11.9)
        self.assertGreater(y.value, 17.3)

        # This problem should have the same solution.
        expr = cp.maximum(cp.floor(cp.ceil(x)), cp.floor(cp.ceil(y)))
        problem = cp.Problem(cp.Minimize(expr), [x >= 12, x <= 17, y >= 17.4])
        self.assertTrue(problem.is_dqcp())
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.objective.value, 18.0)
        self.assertLess(x.value, 17.1)
        self.assertGreater(x.value, 11.9)
        self.assertGreater(y.value, 17.3)

    def test_basic_floor(self):
        x = cp.Variable()
        expr = cp.floor(x)

        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconvex())
        self.assertTrue(expr.is_quasiconcave())
        self.assertFalse(expr.is_convex())
        self.assertFalse(expr.is_concave())
        self.assertFalse(expr.is_dcp())
        self.assertFalse(expr.is_dgp())

        problem = cp.Problem(cp.Minimize(expr), [x >= 11.8, x <= 17])
        self.assertTrue(problem.is_dqcp())
        self.assertFalse(problem.is_dcp())
        self.assertFalse(problem.is_dgp())

        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.objective.value, 11.0)
        self.assertGreater(x.value, 11.7)

    def test_basic_multiply_nonneg(self):
        x, y = cp.Variable(2, nonneg=True)
        expr = x * y
        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconcave())
        self.assertFalse(expr.is_quasiconvex())

        self.assertFalse(expr.is_dcp())

        problem = cp.Problem(cp.Maximize(expr), [x <= 12, y <= 6])
        self.assertTrue(problem.is_dqcp())
        self.assertFalse(problem.is_dcp())
        self.assertFalse(problem.is_dgp())

        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, 72, places=1)
        self.assertAlmostEqual(x.value, 12, places=1)
        self.assertAlmostEqual(y.value, 6, places=1)

    def test_basic_multiply_nonpos(self):
        x, y = cp.Variable(2, nonpos=True)
        expr = x * y
        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconcave())
        self.assertFalse(expr.is_quasiconvex())

        self.assertFalse(expr.is_dcp())

        problem = cp.Problem(cp.Maximize(expr), [x >= -12, y >= -6])
        self.assertTrue(problem.is_dqcp())
        self.assertFalse(problem.is_dcp())
        self.assertFalse(problem.is_dgp())

        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, 72, places=1)
        self.assertAlmostEqual(x.value, -12, places=1)
        self.assertAlmostEqual(y.value, -6, places=1)

    def test_basic_multiply_qcvx(self):
        x = cp.Variable(nonneg=True)
        y = cp.Variable(nonpos=True)
        expr = x * y
        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconvex())
        self.assertFalse(expr.is_quasiconcave())

        self.assertFalse(expr.is_dcp())

        problem = cp.Problem(cp.Minimize(expr), [x <= 7, y >= -6])
        self.assertTrue(problem.is_dqcp())
        self.assertFalse(problem.is_dcp())
        self.assertFalse(problem.is_dgp())

        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, -42, places=1)
        self.assertAlmostEqual(x.value, 7, places=1)
        self.assertAlmostEqual(y.value, -6, places=1)

        x = cp.Variable(nonneg=True)
        y = cp.Variable(nonpos=True)
        expr = y * x
        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconvex())
        self.assertFalse(expr.is_quasiconcave())

        self.assertFalse(expr.is_dcp())

        problem = cp.Problem(cp.Minimize(expr), [x <= 7, y >= -6])
        self.assertTrue(problem.is_dqcp())
        self.assertFalse(problem.is_dcp())
        self.assertFalse(problem.is_dgp())

        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, -42, places=1)
        self.assertAlmostEqual(x.value, 7, places=1)
        self.assertAlmostEqual(y.value, -6, places=1)

    def test_concave_multiply(self):
        x, y = cp.Variable(2, nonneg=True)
        expr = cp.sqrt(x) * cp.sqrt(y)
        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconcave())
        self.assertFalse(expr.is_quasiconvex())

        problem = cp.Problem(cp.Maximize(expr), [x <= 4, y <= 9])
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, 6, places=1)
        self.assertAlmostEqual(x.value, 4, places=1)
        self.assertAlmostEqual(y.value, 9, places=1)

        x, y = cp.Variable(2, nonneg=True)
        expr = (cp.sqrt(x) + 2.0) * (cp.sqrt(y) + 4.0)
        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconcave())
        self.assertFalse(expr.is_quasiconvex())

        problem = cp.Problem(cp.Maximize(expr), [x <= 4, y <= 9])
        problem.solve(SOLVER, qcp=True)
        # (2 + 2) * (3 + 4) = 28
        self.assertAlmostEqual(problem.objective.value, 28, places=1)
        self.assertAlmostEqual(x.value, 4, places=1)
        self.assertAlmostEqual(y.value, 9, places=1)

    def test_basic_ratio(self):
        x = cp.Variable()
        y = cp.Variable(nonneg=True)
        expr = x / y
        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconcave())
        self.assertTrue(expr.is_quasiconvex())

        problem = cp.Problem(cp.Minimize(expr), [x == 12, y <= 6])
        self.assertTrue(problem.is_dqcp())

        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, 2.0, places=1)
        self.assertAlmostEqual(x.value, 12, places=1)
        self.assertAlmostEqual(y.value, 6, places=1)

        x = cp.Variable()
        y = cp.Variable(nonpos=True)
        expr = x / y
        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconcave())
        self.assertTrue(expr.is_quasiconvex())

        problem = cp.Problem(cp.Maximize(expr), [x == 12, y >= -6])
        self.assertTrue(problem.is_dqcp())

        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, -2.0, places=1)
        self.assertAlmostEqual(x.value, 12, places=1)
        self.assertAlmostEqual(y.value, -6, places=1)

    def test_lin_frac(self):
        x = cp.Variable((2,), nonneg=True)
        A = np.array([[1.0, 2.0], [3.0, 4.0]])
        b = np.arange(2)
        C = 2 * A
        d = np.arange(2)
        lin_frac = (cp.matmul(A, x) + b) / (cp.matmul(C, x) + d)
        self.assertTrue(lin_frac.is_dqcp())
        self.assertTrue(lin_frac.is_quasiconvex())
        self.assertTrue(lin_frac.is_quasiconcave())

        problem = cp.Problem(cp.Minimize(cp.sum(x)), [x >= 0, lin_frac <= 1])
        self.assertTrue(problem.is_dqcp())
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, 0, places=1)
        np.testing.assert_almost_equal(x.value, 0, decimal=5)

    def test_concave_frac(self):
        x = cp.Variable(nonneg=True)
        concave_frac = cp.sqrt(x) / cp.exp(x)
        self.assertTrue(concave_frac.is_dqcp())
        self.assertTrue(concave_frac.is_quasiconcave())
        self.assertFalse(concave_frac.is_quasiconvex())

        problem = cp.Problem(cp.Maximize(concave_frac))
        self.assertTrue(problem.is_dqcp())
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, 0.428, places=1)
        self.assertAlmostEqual(x.value, 0.5, places=1)

    def test_length(self):
        x = cp.Variable(5)
        expr = cp.length(x)
        self.assertTrue(expr.is_dqcp())
        self.assertTrue(expr.is_quasiconvex())
        self.assertFalse(expr.is_quasiconcave())

        problem = cp.Problem(cp.Minimize(expr), [x[0] == 2.0, x[1] == 1.0])
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.objective.value, 2)
        np.testing.assert_almost_equal(x.value, np.array([2, 1, 0, 0, 0]))

    def test_infeasible(self):
        x = cp.Variable(2)
        problem = cp.Problem(
            cp.Minimize(cp.length(x)), [x == -1, cp.ceil(x) >= 1])
        problem.solve(SOLVER, qcp=True)
        self.assertIn(problem.status, (s.INFEASIBLE, s.INFEASIBLE_INACCURATE))

    def test_sign(self):
        x = cp.Variable()
        problem = cp.Problem(cp.Minimize(cp.sign(x)), [-2 <= x, x <= -0.5])
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.objective.value, -1)
        self.assertLessEqual(x.value, 0)

        problem = cp.Problem(cp.Maximize(cp.sign(x)), [1 <= x, x <= 2])
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.objective.value, 1.0)
        self.assertGreater(x.value, 0)

    def test_dist_ratio(self):
        x = cp.Variable(2)
        a = np.ones(2)
        b = np.zeros(2)
        problem = cp.Problem(cp.Minimize(cp.dist_ratio(x, a, b)), [x <= 0.8])
        problem.solve(SOLVER, qcp=True)
        np.testing.assert_almost_equal(problem.objective.value, 0.25)
        np.testing.assert_almost_equal(x.value, np.array([0.8, 0.8]))

    def test_infeasible_exp_constr(self):
        x = cp.Variable()
        constr = [cp.exp(cp.ceil(x)) <= -5]
        problem = cp.Problem(cp.Minimize(0), constr)
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.status, s.INFEASIBLE)

    def test_infeasible_inv_pos_constr(self):
        x = cp.Variable(nonneg=True)
        constr = [cp.inv_pos(cp.ceil(x)) <= -5]
        problem = cp.Problem(cp.Minimize(0), constr)
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.status, s.INFEASIBLE)

    def test_infeasible_logistic_constr(self):
        x = cp.Variable(nonneg=True)
        constr = [cp.logistic(cp.ceil(x)) <= -5]
        problem = cp.Problem(cp.Minimize(0), constr)
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.status, s.INFEASIBLE)

    def test_noop_exp_constr(self):
        x = cp.Variable()
        constr = [cp.exp(cp.ceil(x)) >= -5]
        problem = cp.Problem(cp.Minimize(0), constr)
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.status, s.OPTIMAL)

    def test_noop_inv_pos_constr(self):
        x = cp.Variable()
        constr = [cp.inv_pos(cp.ceil(x)) >= -5]
        problem = cp.Problem(cp.Minimize(0), constr)
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.status, s.OPTIMAL)

    def test_noop_logistic_constr(self):
        x = cp.Variable(nonneg=True)
        constr = [cp.logistic(cp.ceil(x)) >= -5]
        problem = cp.Problem(cp.Minimize(0), constr)
        problem.solve(SOLVER, qcp=True)
        self.assertEqual(problem.status, s.OPTIMAL)

    def test_gen_lambda_max_matrix_completion(self):
        A = cp.Variable((3, 3))
        B = cp.Variable((3, 3), PSD=True)
        gen_lambda_max = cp.gen_lambda_max(A, B)
        known_indices = tuple(zip(*[[0, 0], [0, 2], [1, 1]]))
        constr = [
          A[known_indices] == [1.0, 1.9, 0.8],
          B[known_indices] == [3.0, 1.4, 0.2],
        ]
        problem = cp.Problem(cp.Minimize(gen_lambda_max), constr)
        self.assertTrue(problem.is_dqcp())
        # smoke test
        problem.solve(cp.SCS, qcp=True)

    def test_card_ls(self):
        n = 10
        np.random.seed(0)
        A = np.random.randn(n, n)
        x_star = np.random.randn(n)
        b = cp.matmul(A, x_star)
        epsilon = 1e-3

        x = cp.Variable(n)
        objective_fn = cp.length(x)
        mse = cp.sum_squares(cp.matmul(A, x) - b)/n
        problem = cp.Problem(cp.Minimize(objective_fn), [mse <= epsilon])
        # smoke test
        problem.solve(SOLVER, qcp=True)

    def test_multiply_const(self):
        x = cp.Variable()
        obj = cp.Minimize(0.5 * cp.ceil(x))
        problem = cp.Problem(obj, [x >= 10])
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(x.value, 10, places=1)
        self.assertAlmostEqual(problem.value, 5, places=1)

        x = cp.Variable()
        obj = cp.Minimize(cp.ceil(x) * 0.5)
        problem = cp.Problem(obj, [x >= 10])
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(x.value, 10, places=1)
        self.assertAlmostEqual(problem.value, 5, places=1)

        x = cp.Variable()
        obj = cp.Maximize(-0.5 * cp.ceil(x))
        problem = cp.Problem(obj, [x >= 10])
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(x.value, 10, places=1)
        self.assertAlmostEqual(problem.value, -5, places=1)

        x = cp.Variable()
        obj = cp.Maximize(cp.ceil(x) * -0.5)
        problem = cp.Problem(obj, [x >= 10])
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(x.value, 10, places=1)
        self.assertAlmostEqual(problem.value, -5, places=1)

    def test_div_const(self):
        x = cp.Variable()
        obj = cp.Minimize(cp.ceil(x) / 0.5)
        problem = cp.Problem(obj, [x >= 10])
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(x.value, 10, places=1)
        self.assertAlmostEqual(problem.value, 20, places=1)

        x = cp.Variable()
        obj = cp.Maximize(cp.ceil(x) / -0.5)
        problem = cp.Problem(obj, [x >= 10])
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(x.value, 10, places=1)
        self.assertAlmostEqual(problem.value, -20, places=1)

    def test_tutorial_example(self):
        x = cp.Variable()
        y = cp.Variable(pos=True)
        objective_fn = -cp.sqrt(x) / y
        problem = cp.Problem(cp.Minimize(objective_fn), [cp.exp(x) <= y])
        # smoke test
        problem.solve(SOLVER, qcp=True)

    def test_curvature(self):
        x = cp.Variable(3)
        expr = cp.length(x)
        self.assertEqual(expr.curvature, s.QUASICONVEX)
        expr = -cp.length(x)
        self.assertEqual(expr.curvature, s.QUASICONCAVE)
        expr = cp.ceil(x)
        self.assertEqual(expr.curvature, s.QUASILINEAR)
        self.assertTrue(expr.is_quasilinear())

    def test_tutorial_dqcp(self):
        # The sign of variables affects curvature analysis.
        x = cp.Variable(nonneg=True)
        concave_frac = x * cp.sqrt(x)
        constraint = [cp.ceil(x) <= 10]
        problem = cp.Problem(cp.Maximize(concave_frac), constraint)
        self.assertTrue(concave_frac.is_quasiconcave())
        self.assertTrue(constraint[0].is_dqcp())
        self.assertTrue(problem.is_dqcp())

        w = cp.Variable()
        fn = w * cp.sqrt(w)
        problem = cp.Problem(cp.Maximize(fn))
        self.assertFalse(fn.is_dqcp())
        self.assertFalse(problem.is_dqcp())

    def test_add_constant(self):
        # The sign of variables affects curvature analysis.
        x = cp.Variable()
        problem = cp.Problem(cp.Minimize(cp.ceil(x) + 5), [x >= 2])
        problem.solve(SOLVER, qcp=True)
        np.testing.assert_almost_equal(x.value, 2)
        np.testing.assert_almost_equal(problem.objective.value, 7)

    def test_max(self):
        x = cp.Variable(2, pos=True)
        obj = cp.max((1 - 2*cp.sqrt(x) + x) / x)
        problem = cp.Problem(cp.Minimize(obj), [x[0] <= 0.5, x[1] <= 0.9])
        self.assertTrue(problem.is_dqcp())
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, 0.1715, places=3)

    def test_min(self):
        x = cp.Variable(2)
        expr = cp.min(cp.ceil(x))
        problem = cp.Problem(cp.Maximize(expr),
                             [x[0] >= 11.9, x[0] <= 15.8, x[1] >= 17.4])
        self.assertTrue(problem.is_dqcp())
        problem.solve(SOLVER, qcp=True)
        self.assertAlmostEqual(problem.objective.value, 16.0)
        self.assertLess(x[0].value, 16.0)
        self.assertGreater(x[0].value, 14.9)
        self.assertGreater(x[1].value, 17.3)

    def test_sum_of_qccv_not_dqcp(self):
        t = cp.Variable(5, pos=True)
        expr = cp.sum(cp.square(t) / t)
        self.assertFalse(expr.is_dqcp())



"""
Copyright 2013 Steven Diamond, Eric Chu

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import cvxpy as cvx
import numpy as np
import cvxpy.settings as s
from cvxpy.tests.base_test import BaseTest
from cvxpy.reductions.solvers.defines \
    import INSTALLED_SOLVERS

MIP_SOLVERS = [cvx.ECOS_BB, cvx.GUROBI, cvx.MOSEK]


class TestMIPVariable(BaseTest):
    """ Unit tests for the expressions/shape module. """

    def setUp(self):
        self.x_bool = cvx.Variable(boolean=True)
        self.y_int = cvx.Variable(integer=True)
        self.A_bool = cvx.Variable((3, 2), boolean=True)
        self.B_int = cvx.Variable((2, 3), integer=True)
        # Check for all installed QP solvers
        self.solvers = [x for x in MIP_SOLVERS if x in INSTALLED_SOLVERS]

    def test_mip_consistency(self):
        """Test that MIP problems are deterministic.
        """
        data_recs = []
        for i in range(5):
            obj = cvx.Minimize(cvx.square(self.y_int - 0.2))
            p = cvx.Problem(obj, [self.A_bool == 0, self.x_bool == self.B_int])
            data_recs.append(p.get_problem_data(cvx.ECOS_BB))

        # Check that problem data and result is always the same.
        for i in range(1, 5):
            # self.assertEqual(result_recs[0], result_recs[i])
            for key in ["c", "A", "b", "G", "h",
                        "bool_vars_idx", "int_vars_idx"]:
                lh_item = data_recs[0][0][key]
                rh_item = data_recs[i][0][key]
                if key in ["A", "G"]:
                    lh_item = lh_item.todense()
                    rh_item = rh_item.todense()
                self.assertItemsAlmostEqual(lh_item, rh_item)

    # def test_mip_print(self):
    #     """Test to string methods for Bool/Int vars.
    #     """
    #     self.assertEqual(repr(self.x_bool), "Bool(1, 1)")
    #     self.assertEqual(repr(self.B_int), "Int(2, 3)")

    def test_all_solvers(self):
        for solver in self.solvers:
            self.bool_prob(solver)
            self.int_prob(solver)
            self.bool_socp(solver)
            self.int_socp(solver)

    def bool_prob(self, solver):
        # Bool in objective.
        obj = cvx.Minimize(cvx.square(self.x_bool - 0.2))
        p = cvx.Problem(obj, [])
        result = p.solve(solver=solver)
        self.assertAlmostEqual(result, 0.04)

        self.assertAlmostEqual(self.x_bool.value, 0)

        # Bool in constraint.
        t = cvx.Variable()
        obj = cvx.Minimize(t)
        p = cvx.Problem(obj, [cvx.square(self.x_bool) <= t])
        result = p.solve(solver=solver)
        self.assertAlmostEqual(result, 0)

        self.assertAlmostEqual(self.x_bool.value, 0, places=4)

        # Matrix Bool in objective.
        C = np.array([[0, 1, 0], [1, 1, 1]]).T
        obj = cvx.Minimize(cvx.sum_squares(self.A_bool - C))
        p = cvx.Problem(obj, [])
        result = p.solve(solver=solver)
        self.assertAlmostEqual(result, 0)

        self.assertItemsAlmostEqual(self.A_bool.value, C, places=4)

        # Matrix Bool in constraint.
        t = cvx.Variable()
        obj = cvx.Minimize(t)
        p = cvx.Problem(obj, [cvx.sum_squares(self.A_bool - C) <= t])
        result = p.solve(solver=solver)
        self.assertAlmostEqual(result, 0)

        self.assertItemsAlmostEqual(self.A_bool.value, C, places=4)

    def int_prob(self, solver):
        # Int in objective.
        obj = cvx.Minimize(cvx.square(self.y_int - 0.2))
        p = cvx.Problem(obj, [])
        result = p.solve(solver=solver)
        self.assertAlmostEqual(result, 0.04)

        self.assertAlmostEqual(self.y_int.value, 0)

        # Infeasible integer problem
        obj = cvx.Minimize(0)
        p = cvx.Problem(obj, [self.y_int == 0.5])
        result = p.solve(solver=solver)
        self.assertEqual(p.status in s.INF_OR_UNB, True)

    def int_socp(self, solver):
        # Int in objective.
        t = cvx.Variable()
        obj = cvx.Minimize(t)
        p = cvx.Problem(obj, [cvx.square(self.y_int - 0.2) <= t])
        result = p.solve(solver=solver)
        self.assertAlmostEqual(result, 0.04)

        self.assertAlmostEqual(self.y_int.value, 0)

    def bool_socp(self, solver):
        # Int in objective.
        t = cvx.Variable()
        obj = cvx.Minimize(t)
        p = cvx.Problem(obj, [cvx.square(self.x_bool - 0.2) <= t])
        result = p.solve(solver=solver)
        self.assertAlmostEqual(result, 0.04)

        self.assertAlmostEqual(self.x_bool.value, 0)



"""
Copyright 2013 Steven Diamond

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from cvxpy.atoms.affine.affine_atom import AffAtom
import cvxpy.lin_ops.lin_utils as lu
import numpy as np


class upper_tri(AffAtom):
    """The vectorized strictly upper triagonal entries.
    """

    def __init__(self, expr):
        super(upper_tri, self).__init__(expr)

    @AffAtom.numpy_numeric
    def numeric(self, values):
        """Vectorize the upper triagonal entries.
        """
        value = np.zeros(self.shape[0])
        count = 0
        for i in range(values[0].shape[0]):
            for j in range(values[0].shape[1]):
                if i < j:
                    value[count] = values[0][i, j]
                    count += 1
        return value

    def validate_arguments(self):
        """Checks that the argument is a square matrix.
        """
        if not self.args[0].ndim == 2 or self.args[0].shape[0] != self.args[0].shape[1]:
            raise ValueError(
                "Argument to upper_tri must be a square matrix."
            )

    def shape_from_args(self):
        """A vector.
        """
        rows, cols = self.args[0].shape
        return (rows*(cols-1)//2, 1)

    def is_atom_log_log_convex(self):
        """Is the atom log-log convex?
        """
        return True

    def is_atom_log_log_concave(self):
        """Is the atom log-log concave?
        """
        return True

    @staticmethod
    def graph_implementation(arg_objs, shape, data=None):
        """Vectorized strictly upper triagonal entries.

        Parameters
        ----------
        arg_objs : list
            LinExpr for each argument.
        shape : tuple
            The shape of the resulting expression.
        data :
            Additional data required by the atom.

        Returns
        -------
        tuple
            (LinOp for objective, list of constraints)
        """
        return (lu.upper_tri(arg_objs[0]), [])



"""
Copyright 2013 Steven Diamond

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from cvxpy.interface.numpy_interface.ndarray_interface import NDArrayInterface
import scipy.sparse as sp
import numpy as np


class MatrixInterface(NDArrayInterface):
    """
    An interface to convert constant values to the numpy matrix class.
    """
    TARGET_MATRIX = np.matrix

    @NDArrayInterface.scalar_const
    def const_to_matrix(self, value, convert_scalars=False):
        """Convert an arbitrary value into a matrix of type self.target_matrix.

        Args:
            value: The constant to be converted.
            convert_scalars: Should scalars be converted?

        Returns:
            A matrix of type self.target_matrix or a scalar.
        """
        # Lists and 1D arrays become column vectors.
        if isinstance(value, list) or \
                isinstance(value, np.ndarray) and value.ndim == 1:
            value = np.asmatrix(value, dtype='float64').T
        # First convert sparse to dense.
        elif sp.issparse(value):
            value = value.todense()
        return np.asmatrix(value, dtype='float64')

    # Return an identity matrix.
    def identity(self, size):
        return np.asmatrix(np.eye(size))

    # A matrix with all entries equal to the given scalar value.
    def scalar_matrix(self, value, shape):
        mat = np.zeros(shape, dtype='float64') + value
        return np.asmatrix(mat)

    def reshape(self, matrix, size):
        return np.reshape(matrix, size, order='F')



"""
Copyright 2013 Steven Diamond and Eric Chu, 2018 Riley Murray.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import cvxpy as cvx
from cvxpy.tests.base_test import BaseTest
import math
import numpy as np
import scipy.linalg as la


class TestSCS(BaseTest):
    """ Unit tests for SuperSCS. """
    def setUp(self):
        self.x = cvx.Variable(2, name='x')
        self.y = cvx.Variable(2, name='y')

        self.A = cvx.Variable((2, 2), name='A')
        self.B = cvx.Variable((2, 2), name='B')
        self.C = cvx.Variable((3, 2), name='C')

    # Overriden method to assume lower accuracy.
    def assertItemsAlmostEqual(self, a, b, places=2):
        super(TestSCS, self).assertItemsAlmostEqual(a, b, places=places)

    # Overriden method to assume lower accuracy.
    def assertAlmostEqual(self, a, b, places=2):
        super(TestSCS, self).assertAlmostEqual(a, b, places=places)

    def test_log_problem(self):
        if cvx.SUPER_SCS in cvx.installed_solvers():
            # Log in objective.
            obj = cvx.Maximize(cvx.sum(cvx.log(self.x)))
            constr = [self.x <= [1, math.e]]
            p = cvx.Problem(obj, constr)
            result = p.solve(solver='SUPER_SCS')
            self.assertAlmostEqual(result, 1)
            self.assertItemsAlmostEqual(self.x.value, [1, math.e])

            # Log in constraint.
            obj = cvx.Minimize(sum(self.x))
            constr = [cvx.log(self.x) >= 0, self.x <= [1, 1]]
            p = cvx.Problem(obj, constr)
            result = p.solve(solver='SUPER_SCS')
            self.assertAlmostEqual(result, 2)
            self.assertItemsAlmostEqual(self.x.value, [1, 1])

    def test_sigma_max(self):
        """Test sigma_max.
        """
        if cvx.SUPER_SCS in cvx.installed_solvers():
            const = cvx.Constant([[1, 2, 3], [4, 5, 6]])
            constr = [self.C == const]
            prob = cvx.Problem(cvx.Minimize(cvx.norm(self.C, 2)), constr)
            result = prob.solve(solver='SUPER_SCS')
            self.assertAlmostEqual(result, cvx.norm(const, 2).value)
            self.assertItemsAlmostEqual(self.C.value, const.value)

    def test_sdp_var(self):
        """Test sdp var.
        """
        if cvx.SUPER_SCS in cvx.installed_solvers():
            const = cvx.Constant([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
            X = cvx.Variable((3, 3), PSD=True)
            prob = cvx.Problem(cvx.Minimize(0), [X == const])
            prob.solve(verbose=True, solver='SUPER_SCS')
            self.assertEqual(prob.status, cvx.INFEASIBLE)

    def test_cplx_mats(self):
        """Test complex matrices.
        """
        if cvx.SUPER_SCS in cvx.installed_solvers():
            # Complex-valued matrix
            K = np.array(np.random.rand(2, 2) + 1j * np.random.rand(2, 2))
            n1 = la.svdvals(K).sum()  # trace norm of K
            # Dual Problem
            X = cvx.Variable((2, 2), complex=True)
            Y = cvx.Variable((2, 2), complex=True)
            # X, Y >= 0 so trace is real
            objective = cvx.Minimize(
                cvx.real(0.5 * cvx.trace(X) + 0.5 * cvx.trace(Y))
            )
            constraints = [
                cvx.bmat([[X, -K.H], [-K, Y]]) >> 0,
                X >> 0,
                Y >> 0,
            ]
            problem = cvx.Problem(objective, constraints)
            sol_scs = problem.solve(solver='SUPER_SCS')
            self.assertEqual(constraints[0].dual_value.shape, (4, 4))
            self.assertEqual(constraints[1].dual_value.shape, (2, 2))
            self.assertEqual(constraints[2].dual_value.shape, (2, 2))
            self.assertAlmostEqual(sol_scs, n1)

    def test_entr(self):
        """Test a problem with entr.
        """
        if cvx.SUPER_SCS in cvx.installed_solvers():
            for n in [5, 10, 25]:
                print(n)
                x = cvx.Variable(n)
                obj = cvx.Maximize(cvx.sum(cvx.entr(x)))
                p = cvx.Problem(obj, [cvx.sum(x) == 1])
                p.solve(solver='SUPER_SCS', verbose=True)
                self.assertItemsAlmostEqual(x.value, n*[1./n])

    def test_exp(self):
        """Test a problem with exp.
        """
        if cvx.SUPER_SCS in cvx.installed_solvers():
            for n in [5, 10, 25]:
                print(n)
                x = cvx.Variable(n)
                obj = cvx.Minimize(cvx.sum(cvx.exp(x)))
                p = cvx.Problem(obj, [cvx.sum(x) == 1])
                p.solve(solver='SUPER_SCS', verbose=True)
                self.assertItemsAlmostEqual(x.value, n*[1./n])

    def test_log(self):
        """Test a problem with log.
        """
        if cvx.SUPER_SCS in cvx.installed_solvers():
            for n in [5, 10, 25]:
                print(n)
                x = cvx.Variable(n)
                obj = cvx.Maximize(cvx.sum(cvx.log(x)))
                p = cvx.Problem(obj, [cvx.sum(x) == 1])
                p.solve(solver='SUPER_SCS', verbose=True)
                self.assertItemsAlmostEqual(x.value, n*[1./n])

    def test_warm_start(self):
        """Test warm starting.
        """
        if cvx.SUPER_SCS in cvx.installed_solvers():
            x = cvx.Variable(10)
            obj = cvx.Minimize(cvx.sum(cvx.exp(x)))
            prob = cvx.Problem(obj, [cvx.sum(x) == 1])
            result = prob.solve(solver='SUPER_SCS', eps=1e-4)
            result2 = prob.solve(solver='SUPER_SCS', warm_start=True, eps=1e-4)
            self.assertAlmostEqual(result2, result, places=2)
